@@ -1,14 +1,13 @@
 /*
- *  $Id: asapinstance.h 748 2005-09-13 08:40:04Z dreibh $
- *
- * RSerPool implementation.
+ * The rsplib Prototype -- An RSerPool Implementation.
+ * Copyright (C) 2005-2006 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
  *
  * Realized in co-operation between Siemens AG
  * and University of Essen, Institute of Computer Networking Technology.
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
  * This program is free software; you can redistribute it and/or
@@ -23,16 +22,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * There are two mailinglists available at http://www.sctp.de/rserpool.html
- * which should be used for any discussion related to this implementation.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * Contact: rsplib-discussion@sctp.de
  *          dreibh@exp-math.uni-essen.de
  *
- * Purpose: ASAP Instance
- *
  */
 
 #ifndef ASAPINSTANCE_H
@@ -41,9 +35,11 @@
 #include "tdtypes.h"
 #include "dispatcher.h"
 #include "tagitem.h"
+#include "messagebuffer.h"
 #include "rserpoolmessage.h"
 #include "poolhandlespacemanagement.h"
 #include "registrartable.h"
+#include "interthreadmessageport.h"
 
 
 #ifdef __cplusplus
@@ -51,12 +47,22 @@ extern "C" {
 #endif
 
 
+struct ASAPInterThreadMessage;
+
 struct ASAPInstance
 {
    struct Dispatcher*                         StateMachine;
 
+   struct InterThreadMessagePort              MainLoopPort;
+   int                                        MainLoopPipe[2];
+   pthread_t                                  MainLoopThread;
+   bool                                       MainLoopShutdown;
+   struct ASAPInterThreadMessage*             LastAITM;
+
    int                                        RegistrarHuntSocket;
+   struct MessageBuffer*                      RegistrarHuntMessageBuffer;
    int                                        RegistrarSocket;
+   struct MessageBuffer*                      RegistrarMessageBuffer;
    RegistrarIdentifierType                    RegistrarIdentifier;
    unsigned long long                         RegistrarConnectionTimeStamp;
 
@@ -68,6 +74,7 @@ struct ASAPInstance
 
    struct FDCallback                          RegistrarHuntFDCallback;
    struct FDCallback                          RegistrarFDCallback;
+   struct Timer                               RegistrarTimeoutTimer;
 
    size_t                                     RegistrarRequestMaxTrials;
    unsigned long long                         RegistrarRequestTimeout;
@@ -75,14 +82,14 @@ struct ASAPInstance
 };
 
 
-#define ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT               5000000
+#define ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT                     0
 #define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_ADDRESS "239.0.0.1:3863"
 #define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_TIMEOUT          5000000
 #define ASAP_DEFAULT_REGISTRAR_CONNECT_MAXTRIALS               3
-#define ASAP_DEFAULT_REGISTRAR_CONNECT_TIMEOUT           1500000
+#define ASAP_DEFAULT_REGISTRAR_CONNECT_TIMEOUT           2500000
 #define ASAP_DEFAULT_REGISTRAR_REQUEST_MAXTRIALS               1
-#define ASAP_DEFAULT_REGISTRAR_REQUEST_TIMEOUT           1500000
-#define ASAP_DEFAULT_REGISTRAR_RESPONSE_TIMEOUT          1500000
+#define ASAP_DEFAULT_REGISTRAR_REQUEST_TIMEOUT           2500000
+#define ASAP_DEFAULT_REGISTRAR_RESPONSE_TIMEOUT          2500000
 
 #define ASAP_BUFFER_SIZE                                   65536
 
@@ -91,11 +98,13 @@ struct ASAPInstance
   * Constructor.
   *
   * @param dispatcher Dispatcher.
+  * @param registrarAnnounceAddress Multicast address for PR announces.
   * @param tags TagItems.
   * @return ASAPInstance or NULL in case of error.
   */
-struct ASAPInstance* asapInstanceNew(struct Dispatcher* dispatcher,
-                                     struct TagItem*    tags);
+struct ASAPInstance* asapInstanceNew(struct Dispatcher*          dispatcher,
+                                     const union sockaddr_union* registrarAnnounceAddress,
+                                     struct TagItem*             tags);
 
 /**
   * Destructor.
@@ -112,10 +121,10 @@ void asapInstanceDelete(struct ASAPInstance* asapInstance);
   * @param poolElement Pool Element.
   * @return RSPERR_OKAY in case of success; error code otherwise.
   */
-unsigned int asapInstanceRegister(
-                struct ASAPInstance*              asapInstance,
-                struct PoolHandle*                poolHandle,
-                struct ST_CLASS(PoolElementNode)* poolElement);
+unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance,
+                                  struct PoolHandle*                poolHandle,
+                                  struct ST_CLASS(PoolElementNode)* poolElementNode,
+                                  const bool                        waitForResponse);
 
 /**
   * Deregister pool element.
@@ -128,7 +137,8 @@ unsigned int asapInstanceRegister(
 unsigned int asapInstanceDeregister(
                 struct ASAPInstance*            asapInstance,
                 struct PoolHandle*              poolHandle,
-                const PoolElementIdentifierType identifier);
+                const PoolElementIdentifierType identifier,
+                const bool                      waitForResponse);
 
 /**
   * Report failure of pool element.
