@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: debug.h 952 2006-02-21 16:49:32Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +29,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "loglevel.h"
+
 
 /*
 #define DEBUG
@@ -37,7 +41,7 @@
 
 
 #ifndef CHECK
-#define CHECK(cond) if(!(cond)) { fprintf(stderr, "INTERNAL ERROR in %s, line %u: condition %s is not satisfied!\n", __FILE__, __LINE__, #cond); abort(); }
+#define CHECK(cond) if(!(cond)) { fprintf(stderr, "INTERNAL ERROR in %s@%s, line %u: condition %s is not satisfied!\n", __FILE__, getHostName(), __LINE__, #cond); abort(); }
 #endif
 
 
