@@ -1,5 +1,5 @@
 /*
- *  $Id: dispatcher.h 464 2004-08-26 09:12:25Z dreibh $
+ *  $Id: dispatcher.h 1083 2006-04-18 10:40:37Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -41,7 +41,9 @@
 #include "tdtypes.h"
 #include "timer.h"
 #include "fdcallback.h"
-#include "leaflinkedredblacktree.h"
+#include "simpleredblacktree.h"
+
+#include <sys/poll.h>
 
 
 #ifdef __cplusplus
@@ -51,25 +53,22 @@ extern "C" {
 
 struct Dispatcher
 {
-   struct LeafLinkedRedBlackTree TimerStorage;
-   struct LeafLinkedRedBlackTree FDCallbackStorage;
-   bool                          AddRemove;
+   struct SimpleRedBlackTree TimerStorage;
+   struct SimpleRedBlackTree FDCallbackStorage;
+   bool                      AddRemove;
 
-   void                          (*Lock)(struct Dispatcher* dispatcher, void* userData);
-   void                          (*Unlock)(struct Dispatcher* dispatcher, void* userData);
-   void*                         LockUserData;
+   void                      (*Lock)(struct Dispatcher* dispatcher, void* userData);
+   void                      (*Unlock)(struct Dispatcher* dispatcher, void* userData);
+   void*                     LockUserData;
 };
 
 
 /**
   * Constructor.
   *
-  * @param lock Lock function (default: dispatcherDefaultLock).
-  * @param unlock Unlock function (default: dispatcherDefaultUnlock).
+  * @param lock Lock function (default: dummy function).
+  * @param unlock Unlock function (default: dummy function).
   * @param lockUserData User data for lock and unlock function.
-  *
-  * @see dispatcherDefaultLock
-  * @see dispatcherDefaultUnlock
   */
 void dispatcherNew(struct Dispatcher* dispatcher,
                    void               (*lock)(struct Dispatcher* dispatcher, void* userData),
@@ -83,16 +82,6 @@ void dispatcherNew(struct Dispatcher* dispatcher,
   */
 void dispatcherDelete(struct Dispatcher* dispatcher);
 
-/**
-  * Default dispatcher lock function. This is a dummy function and does nothing!
-  */
-void dispatcherDefaultLock(struct Dispatcher* dispatcher, void* userData);
-
-/**
-  * Default dispatcher unlock function. This is a dummy function and does nothing!
-  */
-void dispatcherDefaultUnlock(struct Dispatcher* dispatcher, void* userData);
-
 /**
   * Lock dispatcher.
   *
@@ -108,41 +97,37 @@ void dispatcherLock(struct Dispatcher* dispatcher);
 void dispatcherUnlock(struct Dispatcher* dispatcher);
 
 /**
-  * Get select() parameters for user-controlled select() loop.
+  * Get poll() parameters for user-controlled poll() loop.
   *
   * @param dispatcher Dispatcher.
-  * @param n n.
-  * @param readfdset readfdset.
-  * @param writefdset writefdset.
-  * @param exceptfdset exceptfdset.
-  * @param timeout timeout.
+  * @param ufds pollfd array of at least FD_SETSIZE entries.
+  * @param nfds Reference to store number of pollfd entries.
+  * @param timeout Reference to store timeout.
+  * @param pollTimeStamp Reference to store time stamp of dispatcherGetPollParameters() call.
   */
-void dispatcherGetSelectParameters(struct Dispatcher*  dispatcher,
-                                   int*                n,
-                                   fd_set*             readfdset,
-                                   fd_set*             writefdset,
-                                   fd_set*             exceptfdset,
-                                   fd_set*             testfdset,
-                                   unsigned long long* testTS,
-                                   struct timeval*     timeout);
+void dispatcherGetPollParameters(struct Dispatcher*  dispatcher,
+                                 struct pollfd*      ufds,
+                                 unsigned int*       nfds,
+                                 int*                timeout,
+                                 unsigned long long* pollTimeStamp);
 
 /**
-  * Handle results of select() call.
+  * Handle results of poll() call. Important: nfds must be the
+  * value obtained from dispatcherGetPollParameters()!
   *
   * @param dispatcher Dispatcher.
-  * @param result Result value returned by select().
-  * @param readfdset readfdset.
-  * @param writefdset writefdset.
-  * @param exceptfdset exceptfdset.
+  * @param result Result value returned by poll().
+  * @param ufds pollfd array.
+  * @param nfds Number of pollfd entries.
   * @param timeout timeout.
+  * @param pollTimeStamp Time stamp of dispatcherGetPollParameters() call.
   */
-void dispatcherHandleSelectResult(struct Dispatcher*       dispatcher,
-                                  int                      result,
-                                  fd_set*                  readfdset,
-                                  fd_set*                  writefdset,
-                                  fd_set*                  exceptfdset,
-                                  fd_set*                  testfdset,
-                                  const unsigned long long testTS);
+void dispatcherHandlePollResult(struct Dispatcher* dispatcher,
+                                int                result,
+                                struct pollfd*     ufds,
+                                unsigned int       nfds,
+                                int                timeout,
+                                unsigned long long pollTimeStamp);
 
 /**
   * Event loop calling dispatcherGetSelectParameters(), select() and dispatcherHandleSelectResult().
