@@ -1,8 +1,8 @@
 /*
- *  $Id: ext_socket.h 697 2005-08-08 09:26:18Z dreibh $
+ *  $Id: ext_socket.h 1165 2006-07-30 02:07:12Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2006 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
@@ -57,7 +57,7 @@
 
 
 #define SOCKETAPI_MAJOR_VERSION  1
-#define SOCKETAPI_MINOR_VERSION  7000
+#define SOCKETAPI_MINOR_VERSION  9000
 
 
 #define MSG_UNORDERED    (1 << 31)
@@ -80,7 +80,6 @@
 #define SCTP_NOTIFICATION MSG_NOTIFICATION
 #define SCTP_ABORT        MSG_ABORT
 #define SCTP_EOF          MSG_EOF
-#define SCTP_SHUTDOWN     MSG_SHUTDOWN
 #define SCTP_ADDR_OVER    MSG_ADDR_OVER
 #define SCTP_SEND_TO_ALL  MSG_SEND_TO_ALL
 #define SCTP_MULTIADDRS   MSG_MULTIADDRS
@@ -187,13 +186,13 @@ struct sctp_shutdown_event
 };
 
 
-#define SCTP_ADAPTION_INDICATION 6
-struct sctp_adaption_event
+#define SCTP_ADAPTATION_INDICATION 6
+struct sctp_adaptation_event
 {
    uint16_t     sai_type;
    uint16_t     sai_flags;
    uint32_t     sai_length;
-   uint32_t     sai_adaption_ind;
+   uint32_t     sai_adaptation_ind;
    sctp_assoc_t sai_assoc_id;
 };
 
@@ -238,7 +237,7 @@ union sctp_notification {
    struct sctp_remote_error    sn_remote_error;
    struct sctp_send_failed     sn_send_failed;
    struct sctp_shutdown_event  sn_shutdown_event;
-   struct sctp_adaption_event  sn_adaption_event;
+   struct sctp_adaptation_event  sn_adaptation_event;
    struct sctp_pdapi_event     sn_pdapi_event;
 
    struct sctp_data_arrive     sn_data_arrive;
@@ -346,10 +345,17 @@ struct sctp_event_subscribe
    uint8_t sctp_peer_error_event;
    uint8_t sctp_shutdown_event;
    uint8_t sctp_partial_delivery_event;
-   uint8_t sctp_adaption_layer_event;
+   uint8_t sctp_adaptation_layer_event;
 };
 
 
+struct sctp_assoc_value {
+   sctp_assoc_t            assoc_id;
+   uint32_t                assoc_value;
+};
+
+
+
 #define SCTP_INITMSG                1000
 #define SCTP_AUTOCLOSE              1001
 
@@ -365,6 +371,11 @@ struct sctp_event_subscribe
 #define SCTP_NODELAY                1018
 #define SCTP_SET_DEFAULT_SEND_PARAM 1019
 #define SCTP_EVENTS                 1020
+#define SCTP_DELAYED_ACK_TIME       1021
+#define SCTP_FRAGMENT_INTERLEAVE    1022
+#define SCTP_PARTIAL_DELIVERY_POINT 1023
+#define SCTP_MAXSEG                 1024
+#define SCTP_I_WANT_MAPPED_V4_ADDR  1025
 
 
 
@@ -473,6 +484,7 @@ ssize_t sctp_recvmsg(int                     s,
                      socklen_t*              fromlen,
                      struct sctp_sndrcvinfo* sinfo,
                      int*                    msg_flags);
+int ext_pipe(int fds[2]);
 
 
 /**
@@ -530,7 +542,15 @@ int sctp_enableCRC32(const unsigned int enable);
 #define ext_read(a,b,c) ::read(a,b,c)
 #define ext_write(a,b,c) ::write(a,b,c)
 #define ext_select(a,b,c,d,e) ::select(a,b,c,d,e)
-#define ext_poll(a,b,c,d,e) ::poll(a,b,c,d,e)
+#ifdef USE_SELECT
+extern "C" {
+#include <poll.h>
+int ext_poll(struct pollfd* fdlist, long unsigned int count, int time);
+}
+#else
+#define ext_poll(a,b,c) ::poll(a,b,c)
+#endif
+#define ext_pipe(a) ::pipe(a)
 #else
 #define ext_socket(a,b,c) socket(a,b,c)
 #define ext_bind(a,b,c) bind(a,b,c)
@@ -554,7 +574,13 @@ int sctp_enableCRC32(const unsigned int enable);
 #define ext_read(a,b,c) read(a,b,c)
 #define ext_write(a,b,c) write(a,b,c)
 #define ext_select(a,b,c,d,e) select(a,b,c,d,e)
-#define ext_poll(a,b,c,d,e) poll(a,b,c,d,e)
+#ifdef USE_SELECT
+#include <poll.h>
+int ext_poll(struct pollfd* fdlist, long unsigned int count, int time);
+#else
+#define ext_poll(a,b,c) poll(a,b,c)
+#endif
+#define ext_pipe(a) pipe(a)
 #endif
 
 #include <sys/types.h>
@@ -563,7 +589,7 @@ int sctp_enableCRC32(const unsigned int enable);
 #include <sys/uio.h>
 #include <netinet/sctp.h>
 
-
+/*
 #ifndef SCTP_UNORDERED
 #define SCTP_UNORDERED MSG_UNORDERED
 #endif
@@ -576,6 +602,7 @@ int sctp_enableCRC32(const unsigned int enable);
 #ifndef MSG_EOF
 #define SCTP_EOF MSG_EOF
 #endif
+*/
 
 #ifndef SPP_HB_ENABLE
 #define SPP_HB_ENABLE SPP_HB_ENABLED
