@@ -1,5 +1,5 @@
 /*
- *  $Id: loglevel.c 735 2005-08-26 19:32:12Z tuexen $
+ *  $Id: loglevel.c 975 2006-03-01 21:11:38Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -41,21 +41,24 @@
 #include "threadsafety.h"
 #include "stringutilities.h"
 
+#include <sys/utsname.h>
+
 
 #ifdef HAVE_STDERR_FILEPTR
 /* stderr is of type FILE* */
 FILE** gStdLog = &stderr;
 #else
 /* stderr is a macro returning standard error FILE* */
-static FILE* _stderr = stderr;
-FILE**       gStdLog = &_stderr;
+static FILE* _stderr  = stderr;
+FILE**        gStdLog = &_stderr;
 #endif
 
 
-unsigned int        gLogLevel    = LOGLEVEL_ERROR;
-static bool         gColorMode   = true;
-static bool         gCloseStdLog = false;
-struct ThreadSafety gLogMutex;
+unsigned int          gLogLevel      = LOGLEVEL_ERROR;
+static bool           gColorMode     = true;
+static bool           gCloseStdLog   = false;
+struct ThreadSafety   gLogMutex;
+static char           gHostName[128] = { 0x00 };
 
 
 /* ###### Set ASCII color ################################################ */
@@ -112,7 +115,7 @@ bool initLogging(const char* parameter)
       }
    }
    else {
-      printf("ERROR: Invalid logging parameter: %s\n",parameter);
+      fprintf(stderr, "ERROR: Invalid logging parameter %s\n", parameter);
       return(false);
    }
    return(true);
@@ -122,10 +125,19 @@ bool initLogging(const char* parameter)
 /* ###### Begin logging ################################################## */
 void beginLogging()
 {
-   threadSafetyInit(&gLogMutex, "_LogPrinter_");
+   struct utsname hostInfo;
+
+   threadSafetyNew(&gLogMutex, "_LogPrinter_");
    if((gCloseStdLog) && (ftell(*gStdLog) > 0)) {
       fputs("\n#########################################################################################\n\n",*gStdLog);
    }
+   if(uname(&hostInfo) != 0) {
+      strcpy((char*)&gHostName, "?");
+   }
+   else {
+      snprintf((char*)&gHostName, sizeof(gHostName), "%s",
+               hostInfo.nodename);
+   }
    LOG_NOTE
    fprintf(stdlog,"Logging started, log level is %d.\n",gLogLevel);
    LOG_END
@@ -143,7 +155,7 @@ void finishLogging()
       gCloseStdLog = false;
       *gStdLog     = stderr;
    }
-   threadSafetyDestroy(&gLogMutex);
+   threadSafetyDelete(&gLogMutex);
 }
 
 
@@ -159,3 +171,10 @@ void loggingMutexUnlock()
 {
    threadSafetyUnlock(&gLogMutex);
 }
+
+
+/* ###### Get host info string ########################################### */
+const char* getHostName()
+{
+   return((const char*)gHostName);
+}
