@@ -1,5 +1,5 @@
 /*
- *  $Id: loglevel.h 687 2005-08-04 11:19:38Z dreibh $
+ *  $Id: loglevel.h 941 2006-02-17 09:10:13Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -39,6 +39,10 @@
 #ifndef LOGLEVEL_H
 #define LOGLEVEL_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "tdtypes.h"
 #include "timeutilities.h"
 #include "threadsafety.h"
@@ -77,22 +81,23 @@ void setLogColor(const unsigned int color);
 #define logerror(text) fprintf(stdlog,"%s: %s\n", text, strerror(errno))
 
 
-#define LOG_BEGIN(prefix,c1,c2)              \
-   {                                         \
-      loggingMutexLock();                    \
-      setLogColor(c1);                       \
-      printTimeStamp(stdlog);                \
-      setLogColor(c2);                       \
-      fprintf(stdlog,"P%u.%x %s:%u %s()\n",  \
-              (unsigned int)getpid(),        \
-              (unsigned int)pthread_self(),  \
-              __FILE__,                      \
-              __LINE__,                      \
-              __FUNCTION__                   \
-              );                             \
-      setLogColor(c1);                       \
-      printTimeStamp(stdlog);                \
-      setLogColor(c2);                       \
+#define LOG_BEGIN(prefix,c1,c2)                \
+   {                                           \
+      loggingMutexLock();                      \
+      setLogColor(c1);                         \
+      printTimeStamp(stdlog);                  \
+      setLogColor(c2);                         \
+      fprintf(stdlog,"P%u.%x@%s %s:%u %s()\n", \
+              (unsigned int)getpid(),          \
+              (unsigned int)pthread_self(),    \
+              getHostName(),                   \
+              __FILE__,                        \
+              __LINE__,                        \
+              __FUNCTION__                     \
+              );                               \
+      setLogColor(c1);                         \
+      printTimeStamp(stdlog);                  \
+      setLogColor(c2);                         \
       fputs(prefix,stdlog);
 
 #define LOG_END             \
@@ -200,6 +205,11 @@ void loggingMutexLock();
   */
 void loggingMutexUnlock();
 
+/**
+  * Obtain host name.
+  */
+const char* getHostName();
+
 
 #ifdef __cplusplus
 }
