@@ -1,6 +1,6 @@
 /*
  * mutex.cc: PThread mutex class
- * Copyright (C) 2003 by Thomas Dreibholz
+ * Copyright (C) 2003-2006 by Thomas Dreibholz
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,25 +18,18 @@
  *
  */
 
-#include <unistd.h>
-#include <pthread.h>
-
 #include "mutex.h"
 
 
+// ###### Constructor #######################################################
 TDMutex::TDMutex()
 {
-   pthread_mutexattr_t attr;
-   pthread_mutexattr_init(&attr);
-   pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
-
-   pthread_mutex_init(&MyTDMutex, &attr);
-
-   pthread_mutexattr_destroy(&attr);
+   threadSafetyNew(&MyMutex, "TDMutex");
 }
 
 
+// ###### Destructor ########################################################
 TDMutex::~TDMutex()
 {
-   pthread_mutex_destroy(&MyTDMutex);
+   threadSafetyDelete(&MyMutex);
 }
