@@ -1,6 +1,6 @@
 /*
  * mutex.h: PThread mutex class
- * Copyright (C) 2003 by Thomas Dreibholz
+ * Copyright (C) 2003-2006 by Thomas Dreibholz
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #ifndef MUTEX_H
 #define MUTEX_H
 
-#include <pthread.h>
+#include "threadsafety.h"
 
 
 class TDMutex
@@ -31,14 +31,14 @@ class TDMutex
    virtual ~TDMutex();
 
    inline void lock() {
-      pthread_mutex_lock(&MyTDMutex);
+      threadSafetyLock(&MyMutex);
    }
    inline void unlock() {
-      pthread_mutex_unlock(&MyTDMutex);
+      threadSafetyUnlock(&MyMutex);
    }
 
    private:
-   pthread_mutex_t MyTDMutex;
+   struct ThreadSafety MyMutex;
 };
 
 
