@@ -1,15 +1,8 @@
 /*
- *  $Id: netdouble.h 667 2005-07-28 14:15:15Z dreibh $
+ * The rsplib Prototype -- An RSerPool Implementation.
+ * Copyright (C) 2005-2006 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
  *
- * RSerPool implementation.
- *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * $Id: netdouble.h 928 2006-02-10 10:10:30Z dreibh $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,15 +16,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * There are two mailinglists available at http://www.sctp.de/rserpool.html
- * which should be used for any discussion related to this implementation.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * Contact: rsplib-discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
- *
- * Purpose: Network-Transferable Double Datatype
+ *          dreibh@iem.uni-due.de
  *
  */
 
@@ -46,18 +34,7 @@ extern "C" {
 #endif
 
 
-#define NDF_NEGATIVE ((uint16_t)1 << 15)
-#define EXPONENT_BITS 16
-#define MANTISSA_BITS 64
-
-
-typedef struct
-{
-   uint16_t Flags;
-   int16_t  Exponent;
-   uint32_t MantissaHigh;
-   uint32_t MantissaLow;
-} network_double_t;
+typedef unsigned long long network_double_t;
 
 
 network_double_t doubleToNetwork(double value);
