@@ -1,5 +1,5 @@
 /*
- *  $Id: netutilities.c 731 2005-08-25 17:17:42Z tuexen $
+ *  $Id: netutilities.c 1181 2006-08-04 02:19:39Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -8,7 +8,7 @@
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
  * This program is free software; you can redistribute it and/or
@@ -41,16 +41,19 @@
 #include "stringutilities.h"
 #include "randomizer.h"
 
-#include <stdio.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/ioctl.h>
+#include <sys/uio.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <netdb.h>
-#include <sys/ioctl.h>
 #include <net/if.h>
-#include <fcntl.h>
 #include <ext_socket.h>
-#include <sys/uio.h>
+#include <stdio.h>
+#include <fcntl.h>
+#include <netdb.h>
+
 #ifdef SOLARIS
 #include <sys/sockio.h>
 #endif
@@ -143,7 +146,77 @@ ssize_t sctp_sendx(int                           sd,
 #endif
 #endif
 
+#ifdef USE_SELECT
+int ext_poll(struct pollfd* fdlist, long unsigned int count, int time)
+{
+   // ====== Prepare timeout setting ========================================
+   struct timeval  timeout;
+   struct timeval* to;
+   int    fdcount = 0;
+   int tsize;
+   int       result;
+   unsigned int i;
+
+   if(time < 0)
+      to = NULL;
+   else {
+      to = &timeout;
+      timeout.tv_sec  = time / 1000;
+      timeout.tv_usec = (time % 1000) * 1000;
+   }
+
+   // ====== Prepare FD settings ============================================
+   fd_set readfdset;
+   fd_set writefdset;
+   fd_set exceptfdset;
+   FD_ZERO(&readfdset);
+   FD_ZERO(&writefdset);
+   FD_ZERO(&exceptfdset);
+   for(i = 0; i < count; i++) {
+      if(fdlist[i].fd < 0) {
+         continue;
+      }
+      if(fdlist[i].events & POLLIN) {
+         FD_SET(fdlist[i].fd,&readfdset);
+      }
+      if(fdlist[i].events & POLLOUT) {
+         FD_SET(fdlist[i].fd,&writefdset);
+      }
+      FD_SET(fdlist[i].fd,&exceptfdset);
+      fdcount++;
+   }
+   if(fdcount == 0) {
+      return(0);
+   }
+   for(i = 0;i < count;i++) {
+      fdlist[i].revents = 0;
+   }
+
+   // ====== Do ext_select() ================================================
+   tsize  = getdtablesize();
+   result = ext_select(tsize,&readfdset,&writefdset,&exceptfdset,to);
+   if(result < 0) {
+      return(result);
+   }
+
+   // ====== Set result flags ===============================================
+   for(i = 0;i < count;i++) {
+      if(FD_ISSET(fdlist[i].fd,&readfdset) && (fdlist[i].events & POLLIN)) {
+         fdlist[i].revents |= POLLIN;
+      }
+      if(FD_ISSET(fdlist[i].fd,&writefdset) && (fdlist[i].events & POLLOUT)) {
+         fdlist[i].revents |= POLLOUT;
+      }
+      if(FD_ISSET(fdlist[i].fd,&exceptfdset)) {
+         fdlist[i].revents |= POLLERR;
+      }
+   }
+   return(result);
+}
+#endif
+
 #ifdef LINUX
+#define LINUX_PROC_IPV6_FILE "/proc/net/if_inet6"
 #ifdef HAVE_KERNEL_SCTP
 
 #include <netinet/in_systm.h>
@@ -152,8 +225,6 @@ ssize_t sctp_sendx(int                           sd,
 #include <sys/ioctl.h>
 #include <net/if.h>
 
-#define LINUX_PROC_IPV6_FILE "/proc/net/if_inet6"
-
 enum AddressScopingFlags {
    ASF_HideLoopback           = (1 << 0),
    ASF_HideLinkLocal          = (1 << 1),
@@ -618,14 +689,14 @@ size_t getAddressesFromSocket(int sockfd, union sockaddr_union** addressArray)
    }
    else {
       LOG_VERBOSE4
-      fprintf(stdlog, "Obtained %d address(es)\n",addresses);
+      fprintf(stdlog, "Obtained %d address(es)\n",(int)addresses);
       LOG_END
    }
 
    LOG_VERBOSE4
-   fprintf(stdlog, "Obtained addresses: %u\n", addresses);
+   fprintf(stdlog, "Obtained addresses: %u\n", (unsigned int)addresses);
    for(i = 0;i < addresses;i++) {
-      fputaddress((const struct sockaddr*)&addressArray[i], true, stdlog);
+      fputaddress(&(*addressArray)[i].sa, true, stdlog);
       fputs("\n", stdlog);
    }
    LOG_END
@@ -1368,14 +1439,14 @@ int sendtoplus(int                      sockfd,
    setNonBlocking(sockfd);
    if((assocID != 0) || (ppid != 0) || (streamID != 0)) {
       memset(&sri, 0, sizeof(sri));
-      sri.sinfo_assoc_id   = assocID;
-      sri.sinfo_stream     = streamID;
-      sri.sinfo_ppid       = htonl(ppid);
+      sri.sinfo_assoc_id = assocID;
+      sri.sinfo_stream   = streamID;
+      sri.sinfo_ppid     = htonl(ppid);
       /* --- Already resetted to 0 ---
-      sri.sinfo_flags      = 0;
-      sri.sinfo_ssn        = 0;
-      sri.sinfo_tsn        = 0;
-      sri.sinfo_context    = 0;
+      sri.sinfo_flags    = 0;
+      sri.sinfo_ssn      = 0;
+      sri.sinfo_tsn      = 0;
+      sri.sinfo_context  = 0;
       */
       sri.sinfo_timetolive = timeToLive;
 
@@ -1685,6 +1756,167 @@ bool joinOrLeaveMulticastGroup(int                         sd,
 }
 
 
+/* ###### Send multicast message over each possible interface ############ */
+size_t sendMulticastOverAllInterfaces(int                    sd,
+                                      int                    family,
+                                      const void*            buffer,
+                                      const size_t           length,
+                                      const int              flags,
+                                      const struct sockaddr* to,
+                                      socklen_t              tolen)
+{
+   char                lastIFName[IFNAMSIZ];
+   char                ifcbuffer[4096];
+   struct ifconf       ifc;
+   const struct ifreq* ifr;
+   const struct ifreq* ifrbase;
+   int                 offset;
+   struct in_addr      inaddr;
+   unsigned int        outif;
+   bool                ready;
+   size_t              successes = 0;
+
+#ifdef LINUX
+   FILE*               v6list;
+   unsigned int        dummy;
+   char                addrBuffer[36];
+   char                ifName[IFNAMSIZ];
+
+   if(family == AF_INET6) {
+      v6list = fopen(LINUX_PROC_IPV6_FILE, "r");
+      if (v6list != NULL) {
+         lastIFName[0] = 0x00;
+          while(fscanf(v6list, "%32s %02x %02x %02x %02x %32s\n",
+                       (char*)&addrBuffer, &dummy, &dummy, &dummy, &dummy,
+                       (char*)&ifName) == 6) {
+             if(strcmp(lastIFName, ifName)) {
+                strcpy((char*)&lastIFName, ifName);
+                LOG_VERBOSE4
+                fprintf(stdlog, "Trying to send multicast via interface %s...\n",
+                        ifName);
+                LOG_END
+                outif = if_nametoindex(ifName);
+                if(ext_setsockopt(sd, IPPROTO_IPV6, IPV6_MULTICAST_IF, &outif, sizeof(outif)) == 0) {
+                   if(ext_sendto(sd, buffer, length, flags, to, tolen) > 0) {
+                      LOG_VERBOSE
+                      fprintf(stdlog, "Successfully sent multicast via interface %s\n",
+                              ifName);
+                      LOG_END
+                      successes++;
+                   }
+                }
+             }
+          }
+          fclose(v6list);
+      }
+   }
+   else {
+#endif
+
+   /* ====== Get all interfaces ========================================== */
+   ifc.ifc_len = sizeof(ifcbuffer);
+   ifc.ifc_buf = ifcbuffer;
+   if(ext_ioctl(sd, SIOCGIFCONF, (char*)&ifc) >= 0) {
+      /* ====== Loop over all interfaces ================================= */
+      lastIFName[0] = 0x00;
+      ifrbase = ifc.ifc_req;
+      for(offset = 0;offset < ifc.ifc_len; ) {
+         ifr = (const struct ifreq*)(&((char*)ifrbase)[offset]);
+#ifdef HAVE_SIN_LEN
+         /* The ifreq structure is overwritten by the following ioctl
+            calls -> calculate the new offset here! */
+         offset += sizeof(ifr->ifr_name) +
+                      (ifr->ifr_addr.sa_len > sizeof(struct sockaddr) ?
+                         ifr->ifr_addr.sa_len : sizeof(struct sockaddr));
+#else
+         offset += sizeof(struct ifreq);
+#endif
+
+         /* Is the interface's address type the right one? */
+         if(ifr->ifr_addr.sa_family != family) {
+           /* No! */
+           continue;
+         }
+
+         if(strcmp(lastIFName, ifr->ifr_name)) {
+            strcpy((char*)&lastIFName, ifr->ifr_name);
+
+            if(ext_ioctl(sd, SIOCGIFFLAGS, (char*)ifr) >= 0) {
+               if( (ifr->ifr_flags & IFF_UP) &&
+                   (ifr->ifr_flags & IFF_MULTICAST) ) {
+
+                  /* ====== This interface is usable ===================== */
+                  LOG_VERBOSE4
+                  fprintf(stdlog, "Trying to send multicast via interface %s...\n",
+                          ifr->ifr_name);
+                  LOG_END
+
+                  /* ====== Set the interface for multicast output ======= */
+                  ready = false;
+                  if(family == AF_INET) {
+                     if(ext_ioctl(sd, SIOCGIFADDR, (char*)ifr) >= 0) {
+                        memcpy(&inaddr, &((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr, sizeof(struct in_addr));
+                        if(ext_setsockopt(sd, IPPROTO_IP, IP_MULTICAST_IF, &inaddr, sizeof(inaddr)) == 0) {
+                           ready = true;
+                        }
+                        else {
+                           LOG_ERROR
+                           logerror("Cannot set interface using IP_MULTICAST_IF");
+                           LOG_END
+                        }
+                     }
+                     else {
+                        LOG_ERROR
+                        logerror("ioctl SIOCGIFADDR failed - unable to set multicast interface");
+                        fprintf(stdlog, "Interface is %s\n", ifr->ifr_name);
+                        LOG_END
+                     }
+                  }
+#ifndef LINUX
+                  else if(family == AF_INET6) {
+                     outif = if_nametoindex(ifr->ifr_name);
+                     if(ext_setsockopt(sd, IPPROTO_IPV6, IPV6_MULTICAST_IF, &outif, sizeof(outif)) == 0) {
+                        ready = true;
+                     }
+                  }
+#endif
+
+                  /* ====== Actually send the multicast message ========== */
+                  if(ready) {
+                     if(ext_sendto(sd, buffer, length, flags, to, tolen) > 0) {
+                        LOG_VERBOSE5
+                        fprintf(stdlog, "Successfully sent multicast via interface %s\n",
+                                ifr->ifr_name);
+                        LOG_END
+                        successes++;
+                     }
+                  }
+               }
+            }
+         }
+         /*
+         else {
+            LOG_ERROR
+            logerror("ioctl SIOCGIFFLAGS failed - unable to obtain interfaces details");
+            fprintf(stdlog, "Interface is %s\n", ifr->ifr_name);
+            LOG_END
+         }
+         */
+      }
+   }
+   else {
+      LOG_ERROR
+      logerror("ioctl SIOCGIFCONF failed - unable to obtain network interfaces");
+      LOG_END
+   }
+
+#ifdef LINUX
+   }
+#endif
+   return(successes);
+}
+
+
 /* ###### Set SO_REUSEADDR and SO_REUSEPORT (if supported) ############### */
 bool setReusable(int sd, int on)
 {
@@ -2024,9 +2256,11 @@ size_t getladdrsplus(const int              fd,
 #ifdef LINUX
 #ifdef HAVE_KERNEL_SCTP
    uint16_t port;
+   size_t   addrs2;
+   size_t   j;
 #endif
 #endif
-   size_t i;
+   int    i;
 
    if(addrs > 0) {
 #ifdef LINUX
@@ -2040,11 +2274,11 @@ size_t getladdrsplus(const int              fd,
          port = getPort((struct sockaddr*)packedAddresses);
          sctp_freeladdrs(packedAddresses);
 
-         if(obtainLocalAddresses(addressArray, &addrs, ASF_HideMulticast) == true) {
-            for(i = 0;i < addrs;i++) {
-               setPort(&((*addressArray)[i].sa), port);
+         if(obtainLocalAddresses(addressArray, &addrs2, ASF_HideMulticast) == true) {
+            for(j = 0;j < addrs2;j++) {
+               setPort(&((*addressArray)[j].sa), port);
             }
-            return(addrs);
+            return(addrs2);
          }
          return(0);
       }
@@ -2088,17 +2322,46 @@ size_t getpaddrsplus(const int              fd,
 }
 
 
+/* ###### Send SCTP ABORT ################################################ */
+int sendabort(int sockfd, sctp_assoc_t assocID)
+{
+   return(sendtoplus(sockfd, NULL, 0,
+#ifdef MSG_NOSIGNAL
+                     SCTP_ABORT|MSG_NOSIGNAL,
+#else
+                     SCTP_ABORT,
+#endif
+
+                     NULL, 0,
+                     0, assocID, 0, 0, 0));
+}
+
+
+/* ###### Send SCTP SHUTDOWN ############################################# */
+int sendshutdown(int sockfd, sctp_assoc_t assocID)
+{
+   return(sendtoplus(sockfd, NULL, 0,
+#ifdef MSG_NOSIGNAL
+                     SCTP_EOF|MSG_NOSIGNAL,
+#else
+                     SCTP_EOF,
+#endif
+                     NULL, 0,
+                     0, assocID, 0, 0, 0));
+}
+
+
 /* ###### Convert byte order of 64 bit value ############################# */
 static uint64_t byteswap64(const uint64_t value)
 {
-#if (__BYTE_ORDER == __LITTLE_ENDIAN)
+#if BYTE_ORDER == LITTLE_ENDIAN
    const uint32_t a = (uint32_t)(value >> 32);
    const uint32_t b = (uint32_t)(value & 0xffffffff);
    return( (int64_t)((a << 24) | ((a & 0x0000ff00) << 8) |
            ((a & 0x00ff0000) >> 8) | (a >> 24)) |
            ((int64_t)((b << 24) | ((b & 0x0000ff00) << 8) |
            ((b & 0x00ff0000) >> 8) | (b >> 24)) << 32) );
-#elif (__BYTE_ORDER == __BIG_ENDIAN)
+#elif BYTE_ORDER == BIG_ENDIAN
    return(value);
 #else
 #error Byte order undefined!
