@@ -1,5 +1,5 @@
 /*
- *  $Id: netutilities.h 680 2005-08-03 10:40:27Z dreibh $
+ *  $Id: netutilities.h 1167 2006-07-31 02:22:17Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -8,7 +8,7 @@
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
  * This program is free software; you can redistribute it and/or
@@ -328,8 +328,8 @@ int connectplus(int                   sockfd,
   * @param buffer Data to send.
   * @param length Length of data to send.
   * @param flags sendmsg() flags.
-  * @param to Destination address or NULL for connection-oriented socket.
-  * @param tolen Length of destination address or 0 if not given.
+  * @param toaddrs Destination addresses or NULL for connection-oriented socket.
+  * @param toaddrcnt Number of destination addresses.
   * @param ppid SCTP Payload Protocol Identifier.
   * @param assocID SCTP Association ID or 0 for connection-oriented socket.
   * @param streamID SCTP Stream ID.
@@ -376,6 +376,24 @@ int recvfromplus(int                      sockfd,
                  uint16_t*                streamID,
                  const unsigned long long timeout);
 
+/**
+  * Abort SCTP association.
+  *
+  * @param sockfd Socket descriptor.
+  * @param assocID Association ID.
+  * @return 0 in case of success, -1 otherwise.
+  */
+int sendabort(int sockfd, sctp_assoc_t assocID);
+
+/**
+  * Shutdown SCTP association.
+  *
+  * @param sockfd Socket descriptor.
+  * @param assocID Association ID.
+  * @return 0 in case of success, -1 otherwise.
+  */
+int sendshutdown(int sockfd, sctp_assoc_t assocID);
+
 /**
   * Join or leave a multicast group.
   *
@@ -387,6 +405,25 @@ bool joinOrLeaveMulticastGroup(int                         sd,
                                const union sockaddr_union* groupAddress,
                                const bool                  add);
 
+/**
+  * Send multicast message over each possible interface.
+  *
+  * @param sockfd Socket descriptor.
+  * @param family Socket family (AF_INET or AF_INET6).
+  * @param buffer Data to send.
+  * @param length Length of data to send.
+  * @param flags sendmsg() flags.
+  * @param to Destination address or NULL for connection-oriented socket.
+  * @param tolen Length of destination address or 0 if not given.
+  */
+size_t sendMulticastOverAllInterfaces(int                    sockfd,
+                                      int                    family,
+                                      const void*            buffer,
+                                      const size_t           length,
+                                      const int              flags,
+                                      const struct sockaddr* to,
+                                      socklen_t              tolen);
+
 /**
   * Set address and port reuse on socket on or off.
   *
@@ -441,7 +478,7 @@ bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags);
   * @return Point to block of sockaddrs.
   */
 struct sockaddr* pack_sockaddr_union(const union sockaddr_union* addrArray,
-                                     const size_t                   addrs);
+                                     const size_t                addrs);
 
 /**
   * Translate block of sockaddrs into sockaddr_union array.
