@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: peerlist-template.h 954 2006-02-22 12:49:19Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,12 +34,12 @@ extern "C" {
 
 struct ST_CLASS(PeerList)
 {
-   struct ST_CLASSNAME      PeerListIndexStorage;
-   struct ST_CLASSNAME      PeerListTimerStorage;
+   struct ST_CLASSNAME     PeerListIndexStorage;
+   struct ST_CLASSNAME     PeerListTimerStorage;
 
-   RegistrarIdentifierType  OwnIdentifier;
+   RegistrarIdentifierType OwnIdentifier;
 
-   void*                    UserData;
+   void*                   UserData;
 };
 
 
@@ -96,6 +98,9 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListFindNearestNextPeerListNode)(
 struct ST_CLASS(PeerListNode)* ST_CLASS(peerListRemovePeerListNode)(
                                   struct ST_CLASS(PeerList)*        peerList,
                                   struct ST_CLASS(PeerListNode)* peerListNode);
+unsigned int ST_CLASS(peerListCheckPeerListNodeCompatibility)(
+                const struct ST_CLASS(PeerList)*     peerList,
+                const struct ST_CLASS(PeerListNode)* peerListNode);
 void ST_CLASS(peerListGetDescription)(struct ST_CLASS(PeerList)* peerList,
                                       char*                      buffer,
                                       const size_t               bufferSize);
