@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: peerlist-template_impl.h 954 2006-02-22 12:49:19Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -269,12 +271,14 @@ void ST_CLASS(peerListClear)(struct ST_CLASS(PeerList)* peerList,
 
 /* ###### Check, if node may be inserted into pool ####################### */
 unsigned int ST_CLASS(peerListCheckPeerListNodeCompatibility)(
-                struct ST_CLASS(PeerList)*     peerList,
-                struct ST_CLASS(PeerListNode)* peerListNode)
+                const struct ST_CLASS(PeerList)*     peerList,
+                const struct ST_CLASS(PeerListNode)* peerListNode)
 {
+   /* It does not make sense to insert its one's own ID into the
+      peer list! */
    if((peerList->OwnIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER) &&
       (peerListNode->Identifier == peerList->OwnIdentifier)) {
-      return(RSPERR_INVALID_ID);
+      return(RSPERR_OWN_ID);
    }
    return(RSPERR_OKAY);
 }
@@ -314,13 +318,10 @@ void ST_CLASS(peerListUpdatePeerListNode)(
 {
    struct STN_CLASSNAME* result;
 
-   *errorCode = ST_CLASS(peerListCheckPeerListNodeCompatibility)(peerList, peerListNode);
+   *errorCode = ST_CLASS(peerListCheckPeerListNodeCompatibility)(peerList, source);
    if(*errorCode == RSPERR_OKAY) {
-      /* Update ID: turn 0 -> ID or turn ID -> 0 */
+      /* Update ID */
       if(peerListNode->Identifier != source->Identifier) {
-         CHECK((peerListNode->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER) ||
-               (source->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER));
-
          result = ST_METHOD(Remove)(&peerList->PeerListIndexStorage,
                                     &peerListNode->PeerListIndexStorageNode);
          CHECK(result == &peerListNode->PeerListIndexStorageNode);
@@ -369,18 +370,31 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListAddOrUpdatePeerListNode)(
 /* ###### Find PeerListNode ############################################## */
 struct ST_CLASS(PeerListNode)* ST_CLASS(peerListFindPeerListNode)(
                                   struct ST_CLASS(PeerList)*          peerList,
-                                  const RegistrarIdentifierType            identifier,
+                                  const RegistrarIdentifierType       identifier,
                                   const struct TransportAddressBlock* transportAddressBlock)
 {
-   struct ST_CLASS(PeerListNode) cmpElement;
-   struct STN_CLASSNAME*         result;
+   struct ST_CLASS(PeerListNode)* peerListNode;
+   struct ST_CLASS(PeerListNode)  cmpElement;
+   struct STN_CLASSNAME*          result;
 
-   cmpElement.Identifier   = identifier;
-   cmpElement.AddressBlock = (struct TransportAddressBlock*)transportAddressBlock;
-   result = ST_METHOD(Find)(&peerList->PeerListIndexStorage,
-                            &cmpElement.PeerListIndexStorageNode);
-   if(result) {
-      return(ST_CLASS(getPeerListNodeFromPeerListIndexStorageNode)(result));
+   if(identifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
+      cmpElement.Identifier   = identifier;
+      cmpElement.AddressBlock = (struct TransportAddressBlock*)transportAddressBlock;
+      result = ST_METHOD(Find)(&peerList->PeerListIndexStorage,
+                               &cmpElement.PeerListIndexStorageNode);
+      if(result) {
+         return(ST_CLASS(getPeerListNodeFromPeerListIndexStorageNode)(result));
+      }
+   }
+   else {
+      peerListNode = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(peerList);
+      while(peerListNode != NULL) {
+         if(transportAddressBlockOverlapComparison(peerListNode->AddressBlock,
+                                                   transportAddressBlock) == 0) {
+            return(peerListNode);
+         }
+         peerListNode = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(peerList, peerListNode);
+      }
    }
    return(NULL);
 }
@@ -451,9 +465,9 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListRemovePeerListNode)(
 struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetRandomPeerNode)(
                                   struct ST_CLASS(PeerList)* peerList)
 {
-   LeafLinkedTreapNodeValueType       value;
-   const LeafLinkedTreapNodeValueType maxValue = ST_METHOD(GetValueSum)(
-                                                    &peerList->PeerListIndexStorage);
+   unsigned long long       value;
+   const unsigned long long maxValue = ST_METHOD(GetValueSum)(
+                                          &peerList->PeerListIndexStorage);
    if(maxValue < 1) {
       return(NULL);
    }
