@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool List Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: peerlistmanagement-template_impl.h 954 2006-02-22 12:49:19Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -200,36 +202,65 @@ unsigned int ST_CLASS(peerListManagementRegisterPeerListNode)(
    struct TransportAddressBlock* userTransport;
    unsigned int                  errorCode;
 
-   if( ((flags & PLNF_DYNAMIC) && (registrarIdentifier == UNDEFINED_REGISTRAR_IDENTIFIER)) ||
-       ((!(flags & PLNF_DYNAMIC)) && (registrarIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER)) ) {
-      return(RSPERR_INVALID_ID);
-   }
-
-   /* Check, if a static entry is updated with an ID */
-   if(registrarIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
+   /* ====== Handle dynamic entry ======================================== */
+   if(flags & PLNF_DYNAMIC) {
+      if(registrarIdentifier == UNDEFINED_REGISTRAR_IDENTIFIER) {
+         return(RSPERR_INVALID_ID);
+      }
       *peerListNode = ST_CLASS(peerListManagementFindPeerListNode)(peerListManagement,
-                                                                   UNDEFINED_REGISTRAR_IDENTIFIER,
+                                                                   registrarIdentifier,
                                                                    transportAddressBlock);
-      if(*peerListNode) {
-         /* When the found ID is a static entry, update this entry with
-            the ID */
+      if(*peerListNode == NULL) {
+         /* Is this a new dynamic entry or update of static entry?*/
+         *peerListNode = ST_CLASS(peerListManagementFindPeerListNode)(peerListManagement,
+                                                                      UNDEFINED_REGISTRAR_IDENTIFIER,
+                                                                      transportAddressBlock);
+
+         /* ====== New dynamic entry or update of static entry =========== */
+         if(*peerListNode == NULL) {
+            /* New dynamic entry */
+         }
+
+         /* ====== Update static entry =================================== */
+         else {
+            CHECK(!((*peerListNode)->Flags & PLNF_DYNAMIC));
+            ST_CLASS(peerListNodeNew)(&updatedPeerListNode,
+                                      registrarIdentifier,
+                                      (*peerListNode)->Flags, /* PLNF_DYNAMIC is never set here! */
+                                      (*peerListNode)->AddressBlock);
+            ST_CLASS(peerListUpdatePeerListNode)(&peerListManagement->List, *peerListNode,
+                                                 &updatedPeerListNode, &errorCode);
+            return(errorCode);
+         }
+
+      }
+      else {
+         /* ====== Update static entry =================================== */
          if(!((*peerListNode)->Flags & PLNF_DYNAMIC)) {
             ST_CLASS(peerListNodeNew)(&updatedPeerListNode,
-                                    registrarIdentifier,
-                                    (*peerListNode)->Flags, /* PLNF_DYNAMIC is never set here! */
-                                    (*peerListNode)->AddressBlock);
+                                      registrarIdentifier,
+                                      (*peerListNode)->Flags, /* PLNF_DYNAMIC is never set here! */
+                                      (*peerListNode)->AddressBlock);
             ST_CLASS(peerListUpdatePeerListNode)(&peerListManagement->List, *peerListNode,
-                                                &updatedPeerListNode, &errorCode);
-            CHECK(errorCode == RSPERR_OKAY);
-            return(RSPERR_OKAY);
+                                                 &updatedPeerListNode, &errorCode);
+            return(errorCode);
          }
       }
    }
 
+   /* ====== Handle static entry ========================================= */
+   else {
+      if(registrarIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
+         return(RSPERR_INVALID_ID);
+      }
+   }
+
+
+   /* ====== Create or update entry ====================================== */
    if(peerListManagement->NewPeerListNode == NULL) {
       peerListManagement->NewPeerListNode = (struct ST_CLASS(PeerListNode)*)malloc(sizeof(struct ST_CLASS(PeerListNode)));
       if(peerListManagement->NewPeerListNode == NULL) {
-         return(RSPERR_NO_RESOURCES);
+         return(RSPERR_OUT_OF_MEMORY);
       }
    }
 
@@ -265,10 +296,89 @@ unsigned int ST_CLASS(peerListManagementRegisterPeerListNode)(
             peerListManagement,
             *peerListNode);
          *peerListNode = NULL;
-         errorCode = RSPERR_NO_RESOURCES;
+         errorCode = RSPERR_OUT_OF_MEMORY;
       }
    }
 
+
+#if 0
+// ------------------
+
+   if( ((flags & PLNF_DYNAMIC) && (registrarIdentifier == UNDEFINED_REGISTRAR_IDENTIFIER)) ||
+       ((!(flags & PLNF_DYNAMIC)) && (registrarIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER)) ) {
+      return(RSPERR_INVALID_ID);
+   }
+
+   /* Check, if a static entry is updated with an ID */
+   if(registrarIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
+      *peerListNode = ST_CLASS(peerListManagementFindPeerListNode)(peerListManagement,
+                                                                   UNDEFINED_REGISTRAR_IDENTIFIER,
+                                                                   transportAddressBlock);
+printf("FND=%p\n",*peerListNode);
+      if(*peerListNode) {
+         /* When the found ID is a static entry, update this entry with
+            the ID */
+         if(!((*peerListNode)->Flags & PLNF_DYNAMIC)) {
+            ST_CLASS(peerListNodeNew)(&updatedPeerListNode,
+                                    registrarIdentifier,
+                                    (*peerListNode)->Flags, /* PLNF_DYNAMIC is never set here! */
+                                    (*peerListNode)->AddressBlock);
+            ST_CLASS(peerListUpdatePeerListNode)(&peerListManagement->List, *peerListNode,
+                                                &updatedPeerListNode, &errorCode);
+            CHECK(errorCode == RSPERR_OKAY);
+            return(RSPERR_OKAY);
+         }
+      }
+   }
+
+   if(peerListManagement->NewPeerListNode == NULL) {
+      peerListManagement->NewPeerListNode = (struct ST_CLASS(PeerListNode)*)malloc(sizeof(struct ST_CLASS(PeerListNode)));
+      if(peerListManagement->NewPeerListNode == NULL) {
+         return(RSPERR_OUT_OF_MEMORY);
+      }
+   }
+
+   /* Attention: transportAddressBlock MUST be copied when this
+      PeerListNode is added! */
+   ST_CLASS(peerListNodeNew)(peerListManagement->NewPeerListNode,
+                             registrarIdentifier,
+                             flags,
+                             (struct TransportAddressBlock*)transportAddressBlock);
+   *peerListNode = ST_CLASS(peerListAddOrUpdatePeerListNode)(&peerListManagement->List,
+                                                             &peerListManagement->NewPeerListNode,
+                                                             &errorCode);
+   if(errorCode == RSPERR_OKAY) {
+      (*peerListNode)->LastUpdateTimeStamp = currentTimeStamp;
+
+      if(*peerListNode != peerListManagement->NewPeerListNode) {
+puts("UPDATE!=?!!?");
+      }
+
+      userTransport = transportAddressBlockDuplicate(transportAddressBlock);
+      if(userTransport != NULL) {
+         if((*peerListNode)->AddressBlock != transportAddressBlock) {  /* see comment above! */
+            transportAddressBlockDelete((*peerListNode)->AddressBlock);
+            free((*peerListNode)->AddressBlock);
+         }
+         (*peerListNode)->AddressBlock = userTransport;
+
+         if(peerListManagement->Handlespace) {
+            (*peerListNode)->OwnershipChecksum =
+               ST_CLASS(poolHandlespaceNodeComputeOwnershipChecksum)(
+                  &peerListManagement->Handlespace->Handlespace,
+                  (*peerListNode)->Identifier);
+         }
+      }
+      else {
+         ST_CLASS(peerListManagementDeregisterPeerListNodeByPtr)(
+            peerListManagement,
+            *peerListNode);
+         *peerListNode = NULL;
+         errorCode = RSPERR_OUT_OF_MEMORY;
+      }
+   }
+#endif
+
 #ifdef VERIFY
    ST_CLASS(peerListVerify)(&peerListManagement->List);
 #endif
