@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: peerlistnode-template_impl.h 704 2006-02-22 09:04:55Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,18 +38,6 @@ int ST_CLASS(peerListIndexStorageNodeComparison)(const void* nodePtr1,
    const struct ST_CLASS(PeerListNode)* node2 = (struct ST_CLASS(PeerListNode)*)nodePtr2;
    int                                  result;
 
-   /* One of the nodes has an undefined ID, i.e. static entry.
-      => Only comparison by address overlap possible */
-   if((node1->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER) ||
-      (node2->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER)) {
-      result = transportAddressBlockOverlapComparison(node1->AddressBlock,
-                                                      node2->AddressBlock);
-      if(result == 0) {
-         return(0);
-      }
-      /* No overlap -> continue comparison */
-   }
-
    /* Compare IDs */
    if(node1->Identifier < node2->Identifier) {
       return(-1);
@@ -64,7 +54,6 @@ int ST_CLASS(peerListIndexStorageNodeComparison)(const void* nodePtr1,
                                                node2->AddressBlock);
       return(result);
    }
-
    return(0);
 }
 
@@ -192,6 +181,9 @@ void ST_CLASS(peerListNodeGetDescription)(
    else {
       safestrcat(buffer, "dynamic", bufferSize);
    }
+   if(peerListNode->Flags & PLNF_FROM_PEER) {
+      safestrcat(buffer, "+fromPeer", bufferSize);
+   }
    if(peerListNode->Flags & PLNF_MULTICAST) {
       safestrcat(buffer, "+multicast", bufferSize);
    }
