@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolelementnode-template.h 1022 2006-03-16 09:00:38Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,14 +91,14 @@ void ST_CLASS(poolElementNodeNew)(struct ST_CLASS(PoolElementNode)* poolElementN
                                   const sctp_assoc_t                connectionAssocID);
 void ST_CLASS(poolElementNodeDelete)(struct ST_CLASS(PoolElementNode)* poolElementNode);
 void ST_CLASS(poolElementNodeGetDescription)(
-        struct ST_CLASS(PoolElementNode)* poolElementNode,
-        char*                             buffer,
-        const size_t                      bufferSize,
-        const unsigned int                fields);
+        const struct ST_CLASS(PoolElementNode)* poolElementNode,
+        char*                                   buffer,
+        const size_t                            bufferSize,
+        const unsigned int                      fields);
 void ST_CLASS(poolElementNodePrint)(
-        struct ST_CLASS(PoolElementNode)* poolElementNode,
-        FILE*                             fd,
-        const unsigned int                fields);
+        const struct ST_CLASS(PoolElementNode)* poolElementNode,
+        FILE*                                   fd,
+        const unsigned int                      fields);
 HandlespaceChecksumAccumulatorType ST_CLASS(poolElementNodeComputeChecksum)(
                                       const struct ST_CLASS(PoolElementNode)* poolElementNode);
 int ST_CLASS(poolElementNodeUpdate)(struct ST_CLASS(PoolElementNode)*       poolElementNode,
