@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolelementnode-template_impl.h 1077 2006-04-12 18:23:03Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -105,10 +107,10 @@ void ST_CLASS(poolElementNodeDelete)(struct ST_CLASS(PoolElementNode)* poolEleme
 
 /* ###### Get textual description ######################################## */
 void ST_CLASS(poolElementNodeGetDescription)(
-        struct ST_CLASS(PoolElementNode)* poolElementNode,
-        char*                             buffer,
-        const size_t                      bufferSize,
-        const unsigned int                fields)
+        const struct ST_CLASS(PoolElementNode)* poolElementNode,
+        char*                                   buffer,
+        const size_t                            bufferSize,
+        const unsigned int                      fields)
 {
    char tmp[512];
    char poolPolicySettingsDescription[512];
@@ -190,9 +192,9 @@ void ST_CLASS(poolElementNodeGetDescription)(
 
 /* ###### Print ########################################################## */
 void ST_CLASS(poolElementNodePrint)(
-        struct ST_CLASS(PoolElementNode)* poolElementNode,
-        FILE*                             fd,
-        const unsigned int                fields)
+        const struct ST_CLASS(PoolElementNode)* poolElementNode,
+        FILE*                                   fd,
+        const unsigned int                      fields)
 {
    char buffer[4096];
    ST_CLASS(poolElementNodeGetDescription)(poolElementNode,
@@ -308,7 +310,6 @@ int ST_CLASS(poolElementTimerStorageNodeComparison)(const void* nodePtr1, const
 {
    const struct ST_CLASS(PoolElementNode)* node1 = ST_CLASS(getPoolElementNodeFromTimerStorageNode)((void*)nodePtr1);
    const struct ST_CLASS(PoolElementNode)* node2 = ST_CLASS(getPoolElementNodeFromTimerStorageNode)((void*)nodePtr2);
-   int cmpResult;
 
    if(node1->TimerTimeStamp < node2->TimerTimeStamp) {
       return(-1);
@@ -317,17 +318,14 @@ int ST_CLASS(poolElementTimerStorageNodeComparison)(const void* nodePtr1, const
       return(1);
    }
 
-   cmpResult = ST_CLASS(poolIndexStorageNodeComparison)(node1->OwnerPoolNode, node2->OwnerPoolNode);
-   if(cmpResult != 0) {
-      return(cmpResult);
-   }
    if(node1->Identifier < node2->Identifier) {
       return(-1);
    }
    else if(node1->Identifier > node2->Identifier) {
       return(1);
    }
-   return(0);
+
+   return(poolHandleComparison(&node1->OwnerPoolNode->Handle, &node2->OwnerPoolNode->Handle));
 }
 
 
@@ -344,7 +342,6 @@ int ST_CLASS(poolElementOwnershipStorageNodeComparison)(const void* nodePtr1, co
 {
    const struct ST_CLASS(PoolElementNode)* node1 = ST_CLASS(getPoolElementNodeFromOwnershipStorageNode)((void*)nodePtr1);
    const struct ST_CLASS(PoolElementNode)* node2 = ST_CLASS(getPoolElementNodeFromOwnershipStorageNode)((void*)nodePtr2);
-   int                                     cmpResult;
 
    if(node1->HomeRegistrarIdentifier < node2->HomeRegistrarIdentifier) {
       return(-1);
@@ -352,17 +349,15 @@ int ST_CLASS(poolElementOwnershipStorageNodeComparison)(const void* nodePtr1, co
    else if(node1->HomeRegistrarIdentifier > node2->HomeRegistrarIdentifier) {
       return(1);
    }
-   cmpResult = ST_CLASS(poolIndexStorageNodeComparison)(node1->OwnerPoolNode, node2->OwnerPoolNode);
-   if(cmpResult != 0) {
-      return(cmpResult);
-   }
+
    if(node1->Identifier < node2->Identifier) {
       return(-1);
    }
    else if(node1->Identifier > node2->Identifier) {
       return(1);
    }
-   return(0);
+
+   return(poolHandleComparison(&node1->OwnerPoolNode->Handle, &node2->OwnerPoolNode->Handle));
 }
 
 
@@ -382,7 +377,6 @@ int ST_CLASS(poolElementConnectionStorageNodeComparison)(const void* nodePtr1, c
 {
    const struct ST_CLASS(PoolElementNode)* node1 = ST_CLASS(getPoolElementNodeFromConnectionStorageNode)((void*)nodePtr1);
    const struct ST_CLASS(PoolElementNode)* node2 = ST_CLASS(getPoolElementNodeFromConnectionStorageNode)((void*)nodePtr2);
-   int                                     cmpResult;
 
    if(node1->ConnectionSocketDescriptor < node2->ConnectionSocketDescriptor) {
       return(-1);
@@ -396,17 +390,15 @@ int ST_CLASS(poolElementConnectionStorageNodeComparison)(const void* nodePtr1, c
    else if(node1->ConnectionAssocID > node2->ConnectionAssocID) {
       return(1);
    }
-   cmpResult = ST_CLASS(poolIndexStorageNodeComparison)(node1->OwnerPoolNode, node2->OwnerPoolNode);
-   if(cmpResult != 0) {
-      return(cmpResult);
-   }
+
    if(node1->Identifier < node2->Identifier) {
       return(-1);
    }
    else if(node1->Identifier > node2->Identifier) {
       return(1);
    }
-   return(0);
+
+   return(poolHandleComparison(&node1->OwnerPoolNode->Handle, &node2->OwnerPoolNode->Handle));
 }
 
 
