@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolhandle.c 975 2006-03-01 21:11:38Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,16 +84,12 @@ void poolHandlePrint(const struct PoolHandle* poolHandle,
 int poolHandleComparison(const struct PoolHandle* poolHandle1,
                          const struct PoolHandle* poolHandle2)
 {
-   const int cmpResult = memcmp(poolHandle1->Handle, poolHandle2->Handle,
-                                (poolHandle1->Size < poolHandle2->Size) ? poolHandle1->Size : poolHandle2->Size);
-   if(cmpResult == 0) {
-      if(poolHandle1->Size < poolHandle2->Size) {
-         return(-1);
-      }
-      if(poolHandle1->Size > poolHandle2->Size) {
-         return(1);
-      }
-      return(0);
+   if(poolHandle1->Size < poolHandle2->Size) {
+      return(-1);
    }
-   return(cmpResult);
+   if(poolHandle1->Size > poolHandle2->Size) {
+      return(1);
+   }
+   return(memcmp(poolHandle1->Handle, poolHandle2->Handle,
+                 poolHandle1->Size));
 }
