@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolhandlespacemanagement-template.h 1078 2006-04-13 08:13:02Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,11 +65,11 @@ void ST_CLASS(poolHandlespaceManagementNew)(
 void ST_CLASS(poolHandlespaceManagementDelete)(
         struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
 void ST_CLASS(poolHandlespaceManagementPrint)(
-        struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
-        FILE*                                       fd,
-        const unsigned int                          fields);
+        const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+        FILE*                                             fd,
+        const unsigned int                                fields);
 void ST_CLASS(poolHandlespaceManagementVerify)(
-               struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+        struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
 void ST_CLASS(poolHandlespaceManagementClear)(
         struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
 HandlespaceChecksumType ST_CLASS(poolHandlespaceManagementGetHandlespaceChecksum)(
@@ -135,7 +137,18 @@ unsigned int ST_CLASS(poolHandlespaceManagementHandleResolution)(
                 const size_t                                maxIncrement);
 
 
-#define NTE_MAX_POOL_ELEMENT_NODES 128
+/*
+   Maximum amount of Pool Element Parameters in ASAP/ENRP message:
+   Per PE Transport Parameter:
+   - 20 for Header + PE-ID + Home-PR-ID + Reg.Life
+   - 8+8 for User Transport Parameter with single IPv4 address
+   - 8+8 for ASAP Transport Parameter with single IPv4 address
+   - 8 for Policy Parameter
+   => 60 Bytes.
+   => About ~1090 PE Parameters per message
+      (depending on other overhead, e.g. PH Parameter)
+*/
+#define NTE_MAX_POOL_ELEMENT_NODES 1024
 #define HTEF_START                 (1 << 0)
 #define HTEF_OWNCHILDSONLY         (1 << 1)
 
@@ -152,7 +165,8 @@ int ST_CLASS(poolHandlespaceManagementGetHandleTable)(
        struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
        const RegistrarIdentifierType               homeRegistrarIdentifier,
        struct ST_CLASS(HandleTableExtract)*        handleTableExtract,
-       const unsigned int                          flags);
+       const unsigned int                          flags,
+       size_t                                      maxElements);
 
 
 #ifdef __cplusplus
