@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolhandlespacemanagement-template_impl.h 1091 2006-05-09 08:48:20Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -186,7 +188,7 @@ unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
    if(poolHandlespaceManagement->NewPoolNode == NULL) {
       poolHandlespaceManagement->NewPoolNode = (struct ST_CLASS(PoolNode)*)malloc(sizeof(struct ST_CLASS(PoolNode)));
       if(poolHandlespaceManagement->NewPoolNode == NULL) {
-         return(RSPERR_NO_RESOURCES);
+         return(RSPERR_OUT_OF_MEMORY);
       }
    }
    ST_CLASS(poolNodeNew)(poolHandlespaceManagement->NewPoolNode,
@@ -197,7 +199,7 @@ unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
    if(poolHandlespaceManagement->NewPoolElementNode == NULL) {
       poolHandlespaceManagement->NewPoolElementNode = (struct ST_CLASS(PoolElementNode)*)malloc(sizeof(struct ST_CLASS(PoolElementNode)));
       if(poolHandlespaceManagement->NewPoolElementNode == NULL) {
-         return(RSPERR_NO_RESOURCES);
+         return(RSPERR_OUT_OF_MEMORY);
       }
    }
 
@@ -259,11 +261,12 @@ unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
             poolHandle,
             poolElementIdentifier);
          *poolElementNode = NULL;
-         errorCode = RSPERR_NO_RESOURCES;
+         errorCode = RSPERR_OUT_OF_MEMORY;
       }
    }
 
 #ifdef VERIFY
+#warning VERIFY is on! The Handlespace Management will be very slow!
    ST_CLASS(poolHandlespaceNodeVerify)(&poolHandlespaceManagement->Handlespace);
 #endif
    return(errorCode);
@@ -272,9 +275,9 @@ unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
 
 /* ###### Print handlespace content ######################################## */
 void ST_CLASS(poolHandlespaceManagementPrint)(
-              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
-              FILE*                                       fd,
-              const unsigned int                          fields)
+              const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+              FILE*                                             fd,
+              const unsigned int                                fields)
 {
    ST_CLASS(poolHandlespaceNodePrint)(&poolHandlespaceManagement->Handlespace, fd, fields);
 }
@@ -396,6 +399,7 @@ unsigned int ST_CLASS(poolHandlespaceManagementDeregisterPoolElementByPtr)(
       ST_CLASS(poolHandlespaceManagementPoolNodeDisposer)(poolNode, poolHandlespaceManagement);
    }
 #ifdef VERIFY
+#warning VERIFY is on! The Handlespace Management will be very slow!
       ST_CLASS(poolHandlespaceNodeVerify)(&poolHandlespaceManagement->Handlespace);
 #endif
    return(RSPERR_OKAY);
@@ -438,6 +442,7 @@ unsigned int ST_CLASS(poolHandlespaceManagementHandleResolution)(
                           maxHandleResolutionItems, maxIncrement,
                           &errorCode);
 #ifdef VERIFY
+#warning VERIFY is on! The Handlespace Management will be very slow!
    ST_CLASS(poolHandlespaceNodeVerify)(&poolHandlespaceManagement->Handlespace);
 #endif
 
@@ -464,9 +469,18 @@ static int ST_CLASS(getOwnershipHandleTable)(
               struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
               struct ST_CLASS(HandleTableExtract)*        handleTableExtract,
               const RegistrarIdentifierType               homeRegistrarIdentifier,
-              const unsigned int                          flags)
+              const unsigned int                          flags,
+              size_t                                      maxElements)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNode;
+#ifdef PRINT_GETNAMETABLE_RESULT
+   size_t                            i;
+#endif
+
+   maxElements = min(NTE_MAX_POOL_ELEMENT_NODES, maxElements);
+   if(maxElements < 1) {
+      return(0);
+   }
    if(flags & HTEF_START) {
       poolElementNode = ST_CLASS(poolHandlespaceNodeGetFirstPoolElementOwnershipNodeForIdentifier)(
                            &poolHandlespaceManagement->Handlespace,
@@ -483,7 +497,7 @@ static int ST_CLASS(getOwnershipHandleTable)(
    handleTableExtract->PoolElementNodes = 0;
    while(poolElementNode != NULL) {
       handleTableExtract->PoolElementNodeArray[handleTableExtract->PoolElementNodes++] = poolElementNode;
-      if(handleTableExtract->PoolElementNodes >= NTE_MAX_POOL_ELEMENT_NODES) {
+      if(handleTableExtract->PoolElementNodes >= maxElements) {
          break;
       }
       poolElementNode = ST_CLASS(poolHandlespaceNodeGetNextPoolElementOwnershipNodeForSameIdentifier)(
@@ -498,12 +512,12 @@ static int ST_CLASS(getOwnershipHandleTable)(
    }
 
 #ifdef PRINT_GETNAMETABLE_RESULT
-   printf("GetOwnershipHandleTable result: %u items\n", handleTableExtract->PoolElementNodes);
-   for(size_t i = 0;i < handleTableExtract->PoolElementNodes;i++) {
+   printf("GetOwnershipHandleTable result: %u items\n", (unsigned int)handleTableExtract->PoolElementNodes);
+   for(i = 0;i < handleTableExtract->PoolElementNodes;i++) {
       poolElementNode = handleTableExtract->PoolElementNodeArray[i];
-      printf("#%3d: \"", i);
+      printf("#%3d: \"", (int)i);
       poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdout);
-      printf("\"/%d, $%08x", poolElementNode->OwnerPoolNode->HandleSize, poolElementNode->Identifier);
+      printf("\"/%d, $%08x", (int)poolElementNode->OwnerPoolNode->Handle.Size, poolElementNode->Identifier);
       puts("");
    }
 #endif
@@ -514,11 +528,19 @@ static int ST_CLASS(getOwnershipHandleTable)(
 /* ###### Get name table from handlespace ################################## */
 static int ST_CLASS(getGlobalHandleTable)(struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
                                           struct ST_CLASS(HandleTableExtract)*        handleTableExtract,
-                                          const unsigned int                          flags)
+                                          const unsigned int                          flags,
+                                          size_t                                      maxElements)
 {
    struct ST_CLASS(PoolNode)*        poolNode;
    struct ST_CLASS(PoolElementNode)* poolElementNode;
+#ifdef PRINT_GETNAMETABLE_RESULT
+   size_t                            i;
+#endif
 
+   maxElements = min(NTE_MAX_POOL_ELEMENT_NODES, maxElements);
+   if(maxElements < 1) {
+      return(0);
+   }
    handleTableExtract->PoolElementNodes = 0;
    if(flags & HTEF_START) {
       handleTableExtract->LastPoolElementIdentifier = 0;
@@ -554,7 +576,7 @@ static int ST_CLASS(getGlobalHandleTable)(struct ST_CLASS(PoolHandlespaceManagem
    for(;;) {
       while(poolElementNode != NULL) {
          handleTableExtract->PoolElementNodeArray[handleTableExtract->PoolElementNodes++] = poolElementNode;
-         if(handleTableExtract->PoolElementNodes >= NTE_MAX_POOL_ELEMENT_NODES) {
+         if(handleTableExtract->PoolElementNodes >= maxElements) {
             goto finish;
          }
          poolElementNode = ST_CLASS(poolNodeGetNextPoolElementNodeFromIndex)(poolNode, poolElementNode);
@@ -580,12 +602,12 @@ finish:
 
 
 #ifdef PRINT_GETNAMETABLE_RESULT
-   printf("GetGlobalHandleTable result: %u items\n", handleTableExtract->PoolElementNodes);
-   for(size_t i = 0;i < handleTableExtract->PoolElementNodes;i++) {
+   printf("GetGlobalHandleTable result: %u items\n", (unsigned int)handleTableExtract->PoolElementNodes);
+   for(i = 0;i < handleTableExtract->PoolElementNodes;i++) {
       poolElementNode = handleTableExtract->PoolElementNodeArray[i];
-      printf("#%3d: \"", i);
+      printf("#%3d: \"", (int)i);
       poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdout);
-      printf("\"/%d, $%08x", poolElementNode->OwnerPoolNode->Handle->Size,
+      printf("\"/%d, $%08x", (int)poolElementNode->OwnerPoolNode->Handle.Size,
                              poolElementNode->Identifier);
       puts("");
    }
@@ -599,16 +621,17 @@ int ST_CLASS(poolHandlespaceManagementGetHandleTable)(
        struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
        const RegistrarIdentifierType               homeRegistrarIdentifier,
        struct ST_CLASS(HandleTableExtract)*        handleTableExtract,
-       const unsigned int                          flags)
+       const unsigned int                          flags,
+       size_t                                      maxElements)
 {
    if(flags & HTEF_OWNCHILDSONLY) {
       return(ST_CLASS(getOwnershipHandleTable)(
                 poolHandlespaceManagement, handleTableExtract,
                 homeRegistrarIdentifier,
-                flags));
+                flags, maxElements));
    }
    return(ST_CLASS(getGlobalHandleTable)(
-             poolHandlespaceManagement, handleTableExtract, flags));
+             poolHandlespaceManagement, handleTableExtract, flags, maxElements));
 }
 
 
