@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolhandlespacemanagement.h 934 2006-02-13 15:27:44Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +27,7 @@
 
 
 #include "config.h"
+#include "debug.h"
 #include "rserpoolerror.h"
 #include "poolhandlespacechecksum.h"
 #include "poolhandlespacemanagement-basics.h"
@@ -33,11 +36,28 @@
 #include "transportaddressblock.h"
 #include "stringutilities.h"
 
+
+#ifdef INCLUDE_LINEARLIST
 #include "linearlist.h"
-#include "binarytree.h"
+#endif
+#ifdef INCLUDE_SIMPLEBINARYTREE
+#include "simplebinarytree.h"
+#endif
+#ifdef INCLUDE_LEAFLINKEDBINARYTREE
 #include "leaflinkedbinarytree.h"
+#endif
+#ifdef INCLUDE_SIMPLETREAP
+#include "simpletreap.h"
+#endif
+#ifdef INCLUDE_LEAFLINKEDTREAP
 #include "leaflinkedtreap.h"
+#endif
+#ifdef INCLUDE_SIMPLEREDBLACKTREE
+#include "simpleredblacktree.h"
+#endif
+#ifdef INCLUDE_LEAFLINKEDREDBLACKTREE
 #include "leaflinkedredblacktree.h"
+#endif
 
 
 #define INTERNAL_POOLTEMPLATE
@@ -79,12 +99,12 @@
 
 
 
-#ifdef INCLUDE_BINARYTREE
-#define STN_CLASSNAME BinaryTreeNode
-#define STN_METHOD(x) binaryTreeNode##x
-#define ST_CLASSNAME BinaryTree
-#define ST_CLASS(x) x##_BinaryTree
-#define ST_METHOD(x) binaryTree##x
+#ifdef INCLUDE_SIMPLEBINARYTREE
+#define STN_CLASSNAME SimpleBinaryTreeNode
+#define STN_METHOD(x) simpleBinaryTreeNode##x
+#define ST_CLASSNAME SimpleBinaryTree
+#define ST_CLASS(x) x##_SimpleBinaryTree
+#define ST_METHOD(x) simpleBinaryTree##x
 
 #include "poolpolicy-template.h"
 #include "poolelementnode-template.h"
@@ -153,6 +173,43 @@
 
 
 
+#ifdef INCLUDE_SIMPLETREAP
+#define STN_CLASSNAME SimpleTreapNode
+#define STN_METHOD(x) simpleTreapNode##x
+#define ST_CLASSNAME SimpleTreap
+#define ST_CLASS(x) x##_SimpleTreap
+#define ST_METHOD(x) simpleTreap##x
+
+#include "poolpolicy-template.h"
+#include "poolelementnode-template.h"
+#include "poolnode-template.h"
+#include "poolhandlespacenode-template.h"
+#include "poolhandlespacemanagement-template.h"
+#include "peerlistnode-template.h"
+#include "peerlist-template.h"
+#include "peerlistmanagement-template.h"
+
+#ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
+#include "poolpolicy-template_impl.h"
+#include "poolelementnode-template_impl.h"
+#include "poolnode-template_impl.h"
+#include "poolhandlespacenode-template_impl.h"
+#include "poolhandlespacemanagement-template_impl.h"
+#include "peerlistnode-template_impl.h"
+#include "peerlist-template_impl.h"
+#include "peerlistmanagement-template_impl.h"
+#endif
+
+#undef STN_CLASSNAME
+#undef STN_METHOD
+#undef ST_CLASSNAME
+#undef ST_CLASS
+#undef ST_METHOD
+#endif
+
+
+
+
 #ifdef INCLUDE_LEAFLINKEDTREAP
 #define STN_CLASSNAME LeafLinkedTreapNode
 #define STN_METHOD(x) leafLinkedTreapNode##x
@@ -190,6 +247,43 @@
 
 
 
+#ifdef INCLUDE_SIMPLEREDBLACKTREE
+#define STN_CLASSNAME SimpleRedBlackTreeNode
+#define STN_METHOD(x) simpleRedBlackTreeNode##x
+#define ST_CLASSNAME SimpleRedBlackTree
+#define ST_CLASS(x) x##_SimpleRedBlackTree
+#define ST_METHOD(x) simpleRedBlackTree##x
+
+#include "poolpolicy-template.h"
+#include "poolelementnode-template.h"
+#include "poolnode-template.h"
+#include "poolhandlespacenode-template.h"
+#include "poolhandlespacemanagement-template.h"
+#include "peerlistnode-template.h"
+#include "peerlist-template.h"
+#include "peerlistmanagement-template.h"
+
+#ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
+#include "poolpolicy-template_impl.h"
+#include "poolelementnode-template_impl.h"
+#include "poolnode-template_impl.h"
+#include "poolhandlespacenode-template_impl.h"
+#include "poolhandlespacemanagement-template_impl.h"
+#include "peerlistnode-template_impl.h"
+#include "peerlist-template_impl.h"
+#include "peerlistmanagement-template_impl.h"
+#endif
+
+#undef STN_CLASSNAME
+#undef STN_METHOD
+#undef ST_CLASSNAME
+#undef ST_CLASS
+#undef ST_METHOD
+#endif
+
+
+
+
 #ifdef INCLUDE_LEAFLINKEDREDBLACKTREE
 #define STN_CLASSNAME LeafLinkedRedBlackTreeNode
 #define STN_METHOD(x) leafLinkedRedBlackTreeNode##x
@@ -231,19 +325,19 @@
 #define TMPL_METHOD(x, c) c##x
 
 
-#ifdef USE_TREAP
-#define STN_CLASSNAME LeafLinkedTreapNode
-#define STN_METHOD(x) leafLinkedTreapNode##x
-#define ST_CLASSNAME LeafLinkedTreap
-#define ST_CLASS(x) x##_LeafLinkedTreap
-#define ST_METHOD(x) leafLinkedTreap##x
+#ifdef USE_LIST
+#define STN_CLASSNAME LinearListNode
+#define STN_METHOD(x) linearListNode##x
+#define ST_CLASSNAME LinearList
+#define ST_CLASS(x) x##_LinearList
+#define ST_METHOD(x) linearList##x
 #endif
-#ifdef USE_LEAFLINKEDREDBLACKTREE
-#define STN_CLASSNAME LeafLinkedRedBlackTreeNode
-#define STN_METHOD(x) leafLinkedRedBlackTreeNode##x
-#define ST_CLASSNAME LeafLinkedRedBlackTree
-#define ST_CLASS(x) x##_LeafLinkedRedBlackTree
-#define ST_METHOD(x) leafLinkedRedBlackTree##x
+#ifdef USE_SIMPLEBINARYTREE
+#define STN_CLASSNAME SimpleBinaryTreeNode
+#define STN_METHOD(x) simpleBinaryTreeNode##x
+#define ST_CLASSNAME SimpleBinaryTree
+#define ST_CLASS(x) x##_SimpleBinaryTree
+#define ST_METHOD(x) simpleBinaryTree##x
 #endif
 #ifdef USE_LEAFLINKEDBINARYTREE
 #define STN_CLASSNAME LeafLinkedBinaryTreeNode
@@ -252,19 +346,33 @@
 #define ST_CLASS(x) x##_LeafLinkedBinaryTree
 #define ST_METHOD(x) leafLinkedBinaryTree##x
 #endif
-#ifdef USE_BINARYTREE
-#define STN_CLASSNAME BinaryTreeNode
-#define STN_METHOD(x) binaryTreeNode##x
-#define ST_CLASSNAME BinaryTree
-#define ST_CLASS(x) x##_BinaryTree
-#define ST_METHOD(x) binaryTree##x
+#ifdef USE_SIMPLETREAP
+#define STN_CLASSNAME SimpleTreapNode
+#define STN_METHOD(x) simpleTreapNode##x
+#define ST_CLASSNAME SimpleTreap
+#define ST_CLASS(x) x##_SimpleTreap
+#define ST_METHOD(x) simpleTreap##x
 #endif
-#ifdef USE_LIST
-#define STN_CLASSNAME LinearListNode
-#define STN_METHOD(x) linearListNode##x
-#define ST_CLASSNAME LinearList
-#define ST_CLASS(x) x##_LinearList
-#define ST_METHOD(x) linearList##x
+#ifdef USE_LEAFLINKEDTREAP
+#define STN_CLASSNAME LeafLinkedTreapNode
+#define STN_METHOD(x) leafLinkedTreapNode##x
+#define ST_CLASSNAME LeafLinkedTreap
+#define ST_CLASS(x) x##_LeafLinkedTreap
+#define ST_METHOD(x) leafLinkedTreap##x
+#endif
+#ifdef USE_SIMPLEREDBLACKTREE
+#define STN_CLASSNAME SimpleRedBlackTreeNode
+#define STN_METHOD(x) simpleRedBlackTreeNode##x
+#define ST_CLASSNAME SimpleRedBlackTree
+#define ST_CLASS(x) x##_SimpleRedBlackTree
+#define ST_METHOD(x) simpleRedBlackTree##x
+#endif
+#ifdef USE_LEAFLINKEDREDBLACKTREE
+#define STN_CLASSNAME LeafLinkedRedBlackTreeNode
+#define STN_METHOD(x) leafLinkedRedBlackTreeNode##x
+#define ST_CLASSNAME LeafLinkedRedBlackTree
+#define ST_CLASS(x) x##_LeafLinkedRedBlackTree
+#define ST_METHOD(x) leafLinkedRedBlackTree##x
 #endif
 
 
