@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolhandlespacenode-template.h 1022 2006-03-16 09:00:38Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,7 +74,7 @@ HandlespaceChecksumAccumulatorType ST_CLASS(poolHandlespaceNodeComputeOwnershipC
                                       const struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
                                       const RegistrarIdentifierType               registrarIdentifier);
 size_t ST_CLASS(poolHandlespaceNodeGetTimerNodes)(
-          struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode);
+          const struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode);
 struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceNodeGetFirstPoolElementTimerNode)(
                                      struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode);
 struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceNodeGetLastPoolElementTimerNode)(
@@ -119,7 +121,7 @@ struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceNodeGetNextPoolElement
                                      struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
                                      struct ST_CLASS(PoolElementNode)*     poolElementNode);
 size_t ST_CLASS(poolHandlespaceNodeGetConnectionNodes)(
-          struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode);
+          const struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode);
 size_t ST_CLASS(poolHandlespaceNodeGetConnectionNodesForConnection)(
           struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
           const int                             connectionSocketDescriptor,
@@ -224,12 +226,15 @@ struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceNodeRemovePoolElementN
                                      struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
                                      struct ST_CLASS(PoolElementNode)*     poolElementNode);
 void ST_CLASS(poolHandlespaceNodeGetDescription)(
-        struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
-        char*                                 buffer,
-        const size_t                          bufferSize);
-void ST_CLASS(poolHandlespaceNodePrint)(struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
-                                      FILE*                                   fd,
-                                      const unsigned int                      fields);
+        const struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
+        char*                                       buffer,
+        const size_t                                bufferSize);
+void ST_CLASS(poolHandlespaceNodePrint)(const struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
+                                        FILE*                                       fd,
+                                        const unsigned int                          fields);
+int ST_CLASS(poolHandlespaceNodeHasActiveTimer)(
+       const struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
+       const struct ST_CLASS(PoolElementNode)*     poolElementNode);
 void ST_CLASS(poolHandlespaceNodeActivateTimer)(
         struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode,
         struct ST_CLASS(PoolElementNode)*     poolElementNode,
