@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolnode-template.h 1022 2006-03-16 09:00:38Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -113,12 +115,12 @@ struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeFindNearestNextPoolElementNod
 struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeRemovePoolElementNode)(
                                      struct ST_CLASS(PoolNode)*        poolNode,
                                      struct ST_CLASS(PoolElementNode)* poolElementNode);
-void ST_CLASS(poolNodeGetDescription)(struct ST_CLASS(PoolNode)* poolNode,
-                                      char*                      buffer,
-                                      const size_t               bufferSize);
-void ST_CLASS(poolNodePrint)(struct ST_CLASS(PoolNode)* poolNode,
-                             FILE*                      fd,
-                             const unsigned int         fields);
+void ST_CLASS(poolNodeGetDescription)(const struct ST_CLASS(PoolNode)* poolNode,
+                                      char*                            buffer,
+                                      const size_t                     bufferSize);
+void ST_CLASS(poolNodePrint)(const struct ST_CLASS(PoolNode)* poolNode,
+                             FILE*                            fd,
+                             const unsigned int               fields);
 void ST_CLASS(poolNodeClear)(struct ST_CLASS(PoolNode)* poolNode,
                              void                       (*poolElementNodeDisposer)(void* poolElementNode, void* userData),
                              void*                      userData);
