@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolpolicy-template.h 1079 2006-04-18 08:30:40Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,12 +40,14 @@ struct ST_CLASS(PoolPolicy)
    unsigned int Type;
    const char*  Name;
 
+   size_t       DefaultMaxIncrement;
+
    int (*ComparisonFunction)(const struct ST_CLASS(PoolElementNode)* poolElementNode1,
                              const struct ST_CLASS(PoolElementNode)* poolElementNode2);
    size_t (*SelectionFunction)(struct ST_CLASS(PoolNode)*         poolNode,
                                struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
                                const size_t                       maxPoolElementNodes,
-                               const size_t                       maxIncrement);
+                               size_t                             maxIncrement);
    void (*InitializePoolElementNodeFunction)(struct ST_CLASS(PoolElementNode)* poolElementNode);
    void (*UpdatePoolElementNodeFunction)(struct ST_CLASS(PoolElementNode)* poolElementNode);
    void (*PrepareSelectionFunction)(struct ST_CLASS(PoolNode)* poolNode);
