@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolpolicy-template_impl.h 1092 2006-05-09 12:01:07Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,11 +49,11 @@ static unsigned int ST_CLASS(getSum)(const unsigned int v1,
 
 
 /* ###### Get fraction of base for difference ############################ */
-static LeafLinkedTreapNodeValueType ST_CLASS(getValueFraction)(
-                                       const unsigned int base,
-                                       const unsigned int v1,
-                                       const unsigned int v2,
-                                       const unsigned int v3)
+static unsigned long long ST_CLASS(getValueFraction)(
+                             const unsigned int base,
+                             const unsigned int v1,
+                             const unsigned int v2,
+                             const unsigned int v3)
 {
    const long long base_64 = (long long)base;
    const long long v1_64   = (long long)v1;
@@ -65,12 +67,10 @@ static LeafLinkedTreapNodeValueType ST_CLASS(getValueFraction)(
    else if(v > (long long)0xffffffff) {
       v = (long long)0xffffffff;
    }
-   return((LeafLinkedTreapNodeValueType)v);
+   return((unsigned long long)v);
 }
 
 
-
-
 /*
    #######################################################################
    #### Generic Policy Functions                                      ####
@@ -83,13 +83,17 @@ size_t ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder)(
           struct ST_CLASS(PoolNode)*         poolNode,
           struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
           const size_t                       maxPoolElementNodes,
-          const size_t                       maxIncrement)
+          size_t                             maxIncrement)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNode;
    size_t                            poolElementNodes;
    size_t                            i;
    size_t                            elementsToUpdate;
 
+   /* Set maxIncrement to default, if maxIncrement == 0. */
+   if(maxIncrement == 0) {
+      maxIncrement = poolNode->Policy->DefaultMaxIncrement;
+   }
 
    /* Check, if resequencing is necessary. However, using 64 bit counters,
       this should (almost) never be necessary */
@@ -141,14 +145,18 @@ size_t ST_CLASS(poolPolicySelectPoolElementNodesByValueTree)(
           struct ST_CLASS(PoolNode)*         poolNode,
           struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
           const size_t                       maxPoolElementNodes,
-          const size_t                       maxIncrement)
+          size_t                             maxIncrement)
 {
-   LeafLinkedTreapNodeValueType maxValue;
-   LeafLinkedTreapNodeValueType value;
-   const size_t                 poolElements       = ST_METHOD(GetElements)(&poolNode->PoolElementSelectionStorage);
-   size_t                       poolElementNodes   = 0;
-   size_t                       i;
+   unsigned long long maxValue;
+   unsigned long long value;
+   const size_t       poolElements     = ST_METHOD(GetElements)(&poolNode->PoolElementSelectionStorage);
+   size_t             poolElementNodes = 0;
+   size_t             i;
 
+   /* Set maxIncrement to default, if maxIncrement == 0. */
+   if(maxIncrement == 0) {
+      maxIncrement = poolNode->Policy->DefaultMaxIncrement;
+   }
 
    /* Check, if resequencing is necessary. However, using 64 bit counters,
       this should (almost) never be necessary */
@@ -186,7 +194,9 @@ size_t ST_CLASS(poolPolicySelectPoolElementNodesByValueTree)(
             poolNode->Policy->UpdatePoolElementNodeFunction(poolElementNodeArray[poolElementNodes]);
          }
 
-         ST_CLASS(poolNodeUnlinkPoolElementNodeFromSelection)(poolNode, poolElementNodeArray[poolElementNodes]);
+         if(poolElementNodes < maxIncrement) {
+            ST_CLASS(poolNodeUnlinkPoolElementNodeFromSelection)(poolNode, poolElementNodeArray[poolElementNodes]);
+         }
          poolElementNodes++;
       }
       else {
@@ -194,7 +204,7 @@ size_t ST_CLASS(poolPolicySelectPoolElementNodesByValueTree)(
       }
    }
 
-   for(i = 0;i < poolElementNodes;i++) {
+   for(i = 0;i < min(poolElementNodes, maxIncrement);i++) {
       ST_CLASS(poolNodeLinkPoolElementNodeToSelection)(poolNode, poolElementNodeArray[i]);
    }
 
@@ -352,6 +362,40 @@ static void ST_CLASS(weightedRandomUpdatePoolElementNode)(
 }
 
 
+/*
+   #######################################################################
+   #### Weighted Random with Distance Penalty Factor Policy           ####
+   #######################################################################
+*/
+
+/* ###### Sorting Order ################################################## */
+static int ST_CLASS(weightedRandomDPFComparison)(
+   const struct ST_CLASS(PoolElementNode)* poolElementNode1,
+   const struct ST_CLASS(PoolElementNode)* poolElementNode2)
+{
+   COMPARE_KEY_ASCENDING(poolElementNode1->Identifier, poolElementNode2->Identifier);
+   return(0);
+}
+
+
+/* ###### Update ######################################################### */
+static void ST_CLASS(weightedRandomDPFUpdatePoolElementNode)(
+              struct ST_CLASS(PoolElementNode)* poolElementNode)
+{
+   const double dpf = (double)poolElementNode->PolicySettings.Distance * ((double)poolElementNode->PolicySettings.WeightDPF / (double)0xffffffff);
+   long long value  = (long long)poolElementNode->PolicySettings.Weight -
+                         (long long)rint((double)poolElementNode->PolicySettings.Weight * dpf);
+   if(value < 0) {
+      value = 1;
+   }
+   else if(value > 0xffffffffLL) {
+      value = 0xffffffffLL;
+   }
+
+   poolElementNode->PoolElementSelectionStorageNode.Value = (unsigned int)value;
+}
+
+
 /*
    #######################################################################
    #### Least Used Policy                                             ####
@@ -369,6 +413,50 @@ static int ST_CLASS(leastUsedComparison)(
 }
 
 
+/*
+   #######################################################################
+   #### Least Used with Distance Penalty Factor Policy                ####
+   #######################################################################
+*/
+
+/* ###### Sorting Order ################################################## */
+static int ST_CLASS(leastUsedDPFComparison)(
+   const struct ST_CLASS(PoolElementNode)* poolElementNode1,
+   const struct ST_CLASS(PoolElementNode)* poolElementNode2)
+{
+   const double dpf1 = (double)poolElementNode1->PolicySettings.Distance * ((double)poolElementNode1->PolicySettings.LoadDPF / (double)0xffffffff);
+   const double dpf2 = (double)poolElementNode2->PolicySettings.Distance * ((double)poolElementNode2->PolicySettings.LoadDPF / (double)0xffffffff);
+
+   unsigned long long v1 = (unsigned long long)rint((double)poolElementNode1->PolicySettings.Load + (dpf1 * (double)0xffffff));
+   if(v1 > 0xffffffffULL) {
+      v1 = 0xffffffffULL;
+   }
+   unsigned long long v2 = (unsigned long long)rint((double)poolElementNode2->PolicySettings.Load + (dpf2 * (double)0xffffff));
+   if(v2 > 0xffffffffULL) {
+      v2 = 0xffffffffULL;
+   }
+
+/*
+   printf("dpf1=%1.6lf l1=$%x ldpf1=%1.8lf d1=%3u v1=$%x ; ",
+          dpf1,
+          poolElementNode1->PolicySettings.Load,
+          ((double)poolElementNode1->PolicySettings.LoadDPF / (double)0xffffffff),
+          poolElementNode1->PolicySettings.Distance,
+          v1);
+   printf("dpf2=%1.6lf l2=$%x ldpf2=%1.8lf d2=%3u v2=$%x\n",
+          dpf2,
+          poolElementNode2->PolicySettings.Load,
+          ((double)poolElementNode2->PolicySettings.LoadDPF / (double)0xffffffff),
+          poolElementNode2->PolicySettings.Distance,
+          v2);
+*/
+
+   COMPARE_KEY_ASCENDING(v1, v2);
+   COMPARE_KEY_ASCENDING(poolElementNode1->SeqNumber, poolElementNode2->SeqNumber);
+   return(0);
+}
+
+
 /*
    #######################################################################
    #### Least Used Degradation Policy                                 ####
@@ -401,6 +489,58 @@ static void ST_CLASS(leastUsedDegradationUpdatePoolElementNode)(
 }
 
 
+/*
+   #######################################################################
+   #### Least Used with Degradation & Distance Penalty Factor Policy  ####
+   #######################################################################
+*/
+
+/* ###### Sorting Order ################################################## */
+static int ST_CLASS(leastUsedDegradationDPFComparison)(
+   const struct ST_CLASS(PoolElementNode)* poolElementNode1,
+   const struct ST_CLASS(PoolElementNode)* poolElementNode2)
+{
+   const double dpf1 = (double)poolElementNode1->PolicySettings.Distance * ((double)poolElementNode1->PolicySettings.LoadDPF / (double)0xffffffff);
+   const double dpf2 = (double)poolElementNode2->PolicySettings.Distance * ((double)poolElementNode2->PolicySettings.LoadDPF / (double)0xffffffff);
+
+   unsigned long long v1 = (unsigned long long)rint(
+      (double)poolElementNode1->PolicySettings.Load +
+      (double)poolElementNode1->Degradation +
+      (dpf1 * (double)0xffffff));
+   if(v1 > 0xffffffffULL) {
+      v1 = 0xffffffffULL;
+   }
+   unsigned long long v2 = (unsigned long long)rint(
+      (double)poolElementNode2->PolicySettings.Load +
+      (double)poolElementNode2->Degradation +
+      (dpf2 * (double)0xffffff));
+   if(v2 > 0xffffffffULL) {
+      v2 = 0xffffffffULL;
+   }
+
+/*
+   printf("dpf1=%1.6lf deg1=%1.3f%% l1=$%x ldpf1=%1.8lf d1=%3u v1=$%llx ; ",
+          dpf1,
+          poolElementNode1->Degradation * 100.0 / (double)0xffffff,
+          poolElementNode1->PolicySettings.Load,
+          ((double)poolElementNode1->PolicySettings.LoadDPF / (double)0xffffffff),
+          poolElementNode1->PolicySettings.Distance,
+          v1);
+   printf("dpf2=%1.6lf deg1=%1.3f%% l2=$%x ldpf2=%1.8lf d2=%3u v2=$%llx\n",
+          dpf2,
+          poolElementNode2->Degradation * 100.0 / (double)0xffffff,
+          poolElementNode2->PolicySettings.Load,
+          ((double)poolElementNode2->PolicySettings.LoadDPF / (double)0xffffffff),
+          poolElementNode2->PolicySettings.Distance,
+          v2);
+*/
+
+   COMPARE_KEY_ASCENDING(v1, v2);
+   COMPARE_KEY_ASCENDING(poolElementNode1->SeqNumber, poolElementNode2->SeqNumber);
+   return(0);
+}
+
+
 /*
    #######################################################################
    #### Priority Least Used Policy                                    ####
@@ -570,6 +710,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
 {
    {
       PPT_ROUNDROBIN, "RoundRobin",
+      1,
       &ST_CLASS(roundRobinComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
       NULL,
@@ -578,6 +719,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_WEIGHTED_ROUNDROBIN, "WeightedRoundRobin",
+      1,
       &ST_CLASS(weightedRoundRobinComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
       &ST_CLASS(weightedRoundRobinInitializePoolElementNode),
@@ -586,6 +728,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_RANDOM, "Random",
+      0,
       &ST_CLASS(randomComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
       NULL,
@@ -594,31 +737,62 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_WEIGHTED_RANDOM, "WeightedRandom",
+      0,
       &ST_CLASS(weightedRandomComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
       NULL,
       &ST_CLASS(weightedRandomUpdatePoolElementNode),
       NULL
    },
+   {
+      PPT_WEIGHTED_RANDOM_DPF, "WeightedRandomDPF",
+      0,
+      &ST_CLASS(weightedRandomDPFComparison),
+      &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
+      NULL,
+      &ST_CLASS(weightedRandomDPFUpdatePoolElementNode),
+      NULL
+   },
 
    {
       PPT_LEASTUSED, "LeastUsed",
+      1,
       &ST_CLASS(leastUsedComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
       NULL,
       NULL,
       NULL
    },
+   {
+      PPT_LEASTUSED_DPF, "LeastUsedDPF",
+      1,
+      &ST_CLASS(leastUsedDPFComparison),
+      &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
+      NULL,
+      NULL,
+      NULL
+   },
    {
       PPT_LEASTUSED_DEGRADATION, "LeastUsedDegradation",
+      1,
       &ST_CLASS(leastUsedDegradationComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
       NULL,
       &ST_CLASS(leastUsedDegradationUpdatePoolElementNode),
       NULL
    },
+   {
+      PPT_LEASTUSED_DEGRADATION_DPF, "LeastUsedDegradationDPF",
+      1,
+      &ST_CLASS(leastUsedDegradationDPFComparison),
+      &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
+      NULL,
+      &ST_CLASS(leastUsedDegradationUpdatePoolElementNode),
+      NULL
+   },
    {
       PPT_PRIORITY_LEASTUSED, "PriorityLeastUsed",
+      1,
       &ST_CLASS(priorityLeastUsedComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
       NULL,
@@ -627,6 +801,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_PRIORITY_LEASTUSED_DEGRADATION, "PriorityLeastUsedDegradation",
+      1,
       &ST_CLASS(priorityLeastUsedDegradationComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesBySortingOrder),
       NULL,
@@ -636,6 +811,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
 
    {
       PPT_RANDOMIZED_LEASTUSED, "RandomizedLeastUsed",
+      1,
       &ST_CLASS(randomizedLeastUsedComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
       NULL,
@@ -644,6 +820,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_RANDOMIZED_LEASTUSED_DEGRADATION, "RandomizedLeastUsedDegradation",
+      1,
       &ST_CLASS(randomizedLeastUsedDegradationComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
       NULL,
@@ -652,6 +829,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_RANDOMIZED_PRIORITY_LEASTUSED, "RandomizedPriorityLeastUsed",
+      1,
       &ST_CLASS(randomizedPriorityLeastUsedComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
       NULL,
@@ -660,6 +838,7 @@ const struct ST_CLASS(PoolPolicy) ST_CLASS(PoolPolicyArray)[] =
    },
    {
       PPT_RANDOMIZED_PRIORITY_LEASTUSED_DEGRADATION, "RandomizedPriorityLeastUsedDegradation",
+      1,
       &ST_CLASS(randomizedPriorityLeastUsedDegradationComparison),
       &ST_CLASS(poolPolicySelectPoolElementNodesByValueTree),
       NULL,
