@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolpolicysettings.c 1091 2006-05-09 08:48:20Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,9 @@ void poolPolicySettingsNew(struct PoolPolicySettings* pps)
    pps->Weight          = 0;
    pps->Load            = 0;
    pps->LoadDegradation = 0;
+   pps->LoadDPF         = 0;
+   pps->WeightDPF       = 0;
+   pps->Distance        = 0;
 }
 
 
@@ -40,16 +45,22 @@ void poolPolicySettingsDelete(struct PoolPolicySettings* pps)
    pps->Weight          = 0;
    pps->Load            = 0;
    pps->LoadDegradation = 0;
+   pps->LoadDPF         = 0;
+   pps->WeightDPF       = 0;
+   pps->Distance        = 0;
 }
 
 
 /* ###### Comparison ##################################################### */
 int poolPolicySettingsComparison(const struct PoolPolicySettings* pps1,
-                                        const struct PoolPolicySettings* pps2)
+                                 const struct PoolPolicySettings* pps2)
 {
-   return((pps1->Weight          != pps2->Weight) ||
-          (pps1->Load            != pps2->Load)   ||
-          (pps1->LoadDegradation != pps2->LoadDegradation));
+   return((pps1->Weight          != pps2->Weight)          ||
+          (pps1->Load            != pps2->Load)            ||
+          (pps1->LoadDegradation != pps2->LoadDegradation) ||
+          (pps1->LoadDPF         != pps2->LoadDPF)         ||
+          (pps1->WeightDPF       != pps2->WeightDPF)       ||
+          (pps1->Distance        != pps2->Distance));
 }
 
 
@@ -80,9 +91,14 @@ void poolPolicySettingsGetDescription(const struct PoolPolicySettings* pps,
                                       const size_t                     bufferSize)
 {
    snprintf(buffer, bufferSize,
-            "w=%u l=$%x ldeg=$%x", pps->Weight,
-                                   pps->Load,
-                                   pps->LoadDegradation);
+            "t=$%02x [w=%u l=%1.3f%% ldeg=%1.3f%% ldpf=%1.6f wdpf=%1.6f dist=%u]",
+            pps->PolicyType,
+            pps->Weight,
+            (100.0 * pps->Load) / (double)0xffffff,
+            (100.0 * pps->LoadDegradation) / (double)0xffffff,
+            pps->LoadDPF / (double)0xffffffff,
+            pps->WeightDPF / (double)0xffffffff,
+            pps->Distance);
 }
 
 
