@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2005 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: poolpolicysettings.h 1142 2006-05-24 13:17:42Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,36 +28,23 @@
 
 #include <stdio.h>
 
+#include "rserpool-policytypes.h"
+
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 
-#define PPT_UNDEFINED                                 0x00
-
-#define PPT_ROUNDROBIN                                0x01
-#define PPT_WEIGHTED_ROUNDROBIN                       0x02
-#define PPT_RANDOM                                    0x03
-#define PPT_WEIGHTED_RANDOM                           0x04
-
-#define PPT_LEASTUSED                                 0x05
-#define PPT_LEASTUSED_DEGRADATION                     0x06
-#define PPT_RANDOMIZED_LEASTUSED                      0x07
-#define PPT_RANDOMIZED_LEASTUSED_DEGRADATION          0x08
-
-#define PPT_PRIORITY_LEASTUSED                        0x09
-#define PPT_PRIORITY_LEASTUSED_DEGRADATION            0x0a
-#define PPT_RANDOMIZED_PRIORITY_LEASTUSED             0x0b
-#define PPT_RANDOMIZED_PRIORITY_LEASTUSED_DEGRADATION 0x0c
-
-
 struct PoolPolicySettings
 {
    unsigned int PolicyType;
    unsigned int Weight;
    unsigned int Load;
    unsigned int LoadDegradation;
+   unsigned int LoadDPF;
+   unsigned int WeightDPF;
+   unsigned int Distance;
 };
 
 
@@ -75,7 +64,7 @@ void poolPolicySettingsGetDescription(const struct PoolPolicySettings* pps,
                                       char*                            buffer,
                                       const size_t                     bufferSize);
 int poolPolicySettingsComparison(const struct PoolPolicySettings* pps1,
-                                        const struct PoolPolicySettings* pps2);
+                                 const struct PoolPolicySettings* pps2);
 int poolPolicySettingsIsValid(const struct PoolPolicySettings* pps);
 int poolPolicySettingsAdapt(struct PoolPolicySettings* pps,
                             const unsigned int         destinationType);
