@@ -1,5 +1,5 @@
 /*
- *  $Id: registrartable.c 756 2005-09-16 08:10:02Z dreibh $
+ *  $Id: registrartable.c 1190 2006-08-28 10:14:42Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -8,7 +8,7 @@
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
  * This program is free software; you can redistribute it and/or
@@ -37,15 +37,14 @@
 
 
 #include "tdtypes.h"
-#include "loglevel.h"
 #include "registrartable.h"
-#include "netutilities.h"
-#include "asapinstance.h"
+#include "rserpool-internals.h"
 #include "rserpoolmessage.h"
-#include "rsplib.h"
-#include "rsplib-tags.h"
-#include "randomizer.h"
+#include "asapinstance.h"
 #include "leaflinkedredblacktree.h"
+#include "netutilities.h"
+#include "randomizer.h"
+#include "loglevel.h"
 
 #include <ext_socket.h>
 #include <sys/time.h>
@@ -58,7 +57,7 @@
 
 struct RegistrarAssocIDNode
 {
-   struct LeafLinkedRedBlackTreeNode Node;
+   struct SimpleRedBlackTreeNode Node;
    sctp_assoc_t                      AssocID;
 };
 
@@ -76,7 +75,7 @@ static void handleRegistrarAnnounceCallback(struct RegistrarTable* registrarTabl
    unsigned int                   result;
    size_t                         i;
 
-   LOG_VERBOSE4
+   LOG_VERBOSE
    fputs("Trying to receive registrar announce...\n",  stdlog);
    LOG_END
 
@@ -183,15 +182,15 @@ static int registrarAssocIDNodeComparison(const void* node1, const void* node2)
 static void registrarAssocIDNodePrint(const void* node, FILE* fd)
 {
    const struct RegistrarAssocIDNode* assocIDNode = (const struct RegistrarAssocIDNode*)node;
-   fprintf(fd, "%u ", assocIDNode->AssocID);
+   fprintf(fd, "%u ", (unsigned int)assocIDNode->AssocID);
 }
 
 
 /* ###### Constructor ####################################################### */
-struct RegistrarTable* registrarTableNew(struct Dispatcher* dispatcher,
-                                         struct TagItem*    tags)
+struct RegistrarTable* registrarTableNew(struct Dispatcher*          dispatcher,
+                                         const union sockaddr_union* registrarAnnounceAddress,
+                                         struct TagItem*             tags)
 {
-   union sockaddr_union*  announceAddress;
    union sockaddr_union   defaultAnnounceAddress;
    struct RegistrarTable* registrarTable = (struct RegistrarTable*)malloc(sizeof(struct RegistrarTable));
 
@@ -201,7 +200,7 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher* dispatcher,
       registrarTable->OutstandingConnects = 0;
       registrarTable->AnnounceSocket      = -1;
       ST_CLASS(peerListManagementNew)(&registrarTable->RegistrarList, NULL, 0, NULL, NULL);
-      leafLinkedRedBlackTreeNew(&registrarTable->RegistrarAssocIDList, registrarAssocIDNodePrint, registrarAssocIDNodeComparison);
+      simpleRedBlackTreeNew(&registrarTable->RegistrarAssocIDList, registrarAssocIDNodePrint, registrarAssocIDNodeComparison);
 
 
       /* ====== ASAP Instance settings ==================================== */
@@ -212,21 +211,23 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher* dispatcher,
       registrarTable->RegistrarAnnounceTimeout = (unsigned long long)tagListGetData(tags, TAG_RspLib_RegistrarAnnounceTimeout,
                                                                       ASAP_DEFAULT_REGISTRAR_ANNOUNCE_TIMEOUT);
 
-      CHECK(string2address(ASAP_DEFAULT_REGISTRAR_ANNOUNCE_ADDRESS, &defaultAnnounceAddress) == true);
-      announceAddress = (union sockaddr_union*)tagListGetData(tags, TAG_RspLib_RegistrarAnnounceAddress,
-                                                                 (tagdata_t)&defaultAnnounceAddress);
-      memcpy(&registrarTable->AnnounceAddress, announceAddress, sizeof(registrarTable->AnnounceAddress));
-
+      if(registrarAnnounceAddress) {
+         memcpy(&registrarTable->AnnounceAddress, registrarAnnounceAddress, sizeof(registrarTable->AnnounceAddress));
+      }
+      else {
+         CHECK(string2address(ASAP_DEFAULT_REGISTRAR_ANNOUNCE_ADDRESS, &defaultAnnounceAddress) == true);
+         memcpy(&registrarTable->AnnounceAddress, &defaultAnnounceAddress, sizeof(registrarTable->AnnounceAddress));
+      }
 
       /* ====== Show results ================================================ */
       LOG_VERBOSE3
       fputs("New RegistrarTable's configuration:\n",  stdlog);
-      fprintf(stdlog, "registrar.announce.timeout  = %llu [us]\n",  registrarTable->RegistrarAnnounceTimeout);
+      fprintf(stdlog, "registrar.announce.timeout  = %llu [us]\n", registrarTable->RegistrarAnnounceTimeout);
       fputs("registrar.announce.address  = ",  stdlog);
       fputaddress((struct sockaddr*)&registrarTable->AnnounceAddress, true, stdlog);
       fputs("\n",  stdlog);
       fprintf(stdlog, "registrar.connect.maxtrials = %u\n",        registrarTable->RegistrarConnectMaxTrials);
-      fprintf(stdlog, "registrar.connect.timeout   = %llu [us]\n",  registrarTable->RegistrarConnectTimeout);
+      fprintf(stdlog, "registrar.connect.timeout   = %llu [us]\n", registrarTable->RegistrarConnectTimeout);
       LOG_END
 
 
@@ -253,6 +254,16 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher* dispatcher,
             registrarTableDelete(registrarTable);
             return(false);
          }
+
+         if(!joinOrLeaveMulticastGroup(registrarTable->AnnounceSocket,
+                                       &registrarTable->AnnounceAddress,
+                                       true)) {
+            LOG_WARNING
+            fputs("Joining multicast group ",  stdlog);
+            fputaddress(&registrarTable->AnnounceAddress.sa, true, stdlog);
+            fputs(" failed. Check routing (is default route set?) and firewall settings!\n",  stdlog);
+            LOG_END
+         }
       }
       else {
          LOG_ERROR
@@ -268,61 +279,68 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher* dispatcher,
 
 /* ###### Add registrar assoc ID to list ################################# */
 static void addRegistrarAssocID(struct RegistrarTable* registrarTable,
-                                int                    registrarFD,
+                                int                    registrarHuntFD,
                                 sctp_assoc_t           assocID)
 {
    struct RegistrarAssocIDNode* node = (struct RegistrarAssocIDNode*)malloc(sizeof(struct RegistrarAssocIDNode));
    if(node != NULL) {
-      leafLinkedRedBlackTreeNodeNew(&node->Node);
+      simpleRedBlackTreeNodeNew(&node->Node);
       node->Node.Value = 1;
       node->AssocID    = assocID;
 
-      CHECK(leafLinkedRedBlackTreeInsert(&registrarTable->RegistrarAssocIDList, &node->Node) == &node->Node);
+      CHECK(simpleRedBlackTreeInsert(&registrarTable->RegistrarAssocIDList, &node->Node) == &node->Node);
 
-      LOG_VERBOSE2
-      fprintf(stdlog, "Added assoc %u to registrar assoc ID list.\n" , assocID);
+      LOG_VERBOSE3
+      fprintf(stdlog, "Added assoc %u to registrar assoc ID list.\n" , (unsigned int)assocID);
       fputs("RegistrarAssocIDList: ", stdlog);
-      leafLinkedRedBlackTreePrint(&registrarTable->RegistrarAssocIDList, stdlog);
+      simpleRedBlackTreePrint(&registrarTable->RegistrarAssocIDList, stdlog);
       LOG_END
    }
    else {
-      /* Abort association to current registrar (if existing) */
-      sendtoplus(registrarFD, NULL, 0, SCTP_ABORT, NULL, 0,
-                 0, assocID, 0, 0xffffffff, 0);
+      sendabort(registrarHuntFD, assocID);
    }
 }
 
 
 /* ###### Remove registrar assoc ID to list ############################## */
 static void removeRegistrarAssocID(struct RegistrarTable* registrarTable,
-                                   int                    registrarFD,
+                                   int                    registrarHuntFD,
                                    sctp_assoc_t           assocID)
 {
    struct RegistrarAssocIDNode        cmpNode;
-   struct LeafLinkedRedBlackTreeNode* node;
+   struct SimpleRedBlackTreeNode* node;
 
    cmpNode.AssocID = assocID;
-   node = leafLinkedRedBlackTreeFind(&registrarTable->RegistrarAssocIDList, &cmpNode.Node);
-   CHECK(node != NULL);
-   CHECK(leafLinkedRedBlackTreeRemove(&registrarTable->RegistrarAssocIDList, node) == node);
-   free(node);
+   node = simpleRedBlackTreeFind(&registrarTable->RegistrarAssocIDList, &cmpNode.Node);
+   if(node != NULL) {
+      CHECK(simpleRedBlackTreeRemove(&registrarTable->RegistrarAssocIDList, node) == node);
+      free(node);
 
-   LOG_VERBOSE2
-   fprintf(stdlog, "Removed assoc %u from registrar assoc ID list.\n" , assocID);
-   fputs("RegistrarAssocIDList: ", stdlog);
-   leafLinkedRedBlackTreePrint(&registrarTable->RegistrarAssocIDList, stdlog);
-   LOG_END
+      LOG_VERBOSE3
+      fprintf(stdlog, "Removed assoc %u from registrar assoc ID list.\n" , (unsigned int)assocID);
+      fputs("RegistrarAssocIDList: ", stdlog);
+      simpleRedBlackTreePrint(&registrarTable->RegistrarAssocIDList, stdlog);
+      LOG_END
+   }
+   else {
+      LOG_WARNING
+      fprintf(stderr, "Tried to remove not-existing assoc %u from registrar assoc ID list.\n", (unsigned int)assocID);
+      LOG_END
+   }
 }
 
 
 /* ###### Get registrar assoc ID from list ############################### */
 static sctp_assoc_t selectRegistrarAssocID(struct RegistrarTable* registrarTable)
 {
-   size_t                             elements = leafLinkedRedBlackTreeGetElements(&registrarTable->RegistrarAssocIDList);
-   struct LeafLinkedRedBlackTreeNode* node;
+   size_t                         elements;
+   RedBlackTreeNodeValueType      value;
+   struct SimpleRedBlackTreeNode* node;
 
+   elements = simpleRedBlackTreeGetElements(&registrarTable->RegistrarAssocIDList);
    if(elements > 0) {
-      node = leafLinkedRedBlackTreeGetNodeByValue(&registrarTable->RegistrarAssocIDList, random32() % elements);
+      value = random32() % elements;
+      node = simpleRedBlackTreeGetNodeByValue(&registrarTable->RegistrarAssocIDList, value);
       CHECK(node);
       return(((struct RegistrarAssocIDNode*)node)->AssocID);
    }
@@ -332,7 +350,8 @@ static sctp_assoc_t selectRegistrarAssocID(struct RegistrarTable* registrarTable
 
 /* ###### Get registrar assoc ID from list ############################### */
 static int selectRegistrar(struct RegistrarTable*   registrarTable,
-                           int                      registrarFD,
+                           int                      registrarHuntFD,
+                           struct MessageBuffer*    registrarHuntMessageBuffer,
                            RegistrarIdentifierType* registrarIdentifier)
 {
    sctp_assoc_t                   assocID;
@@ -347,10 +366,10 @@ static int selectRegistrar(struct RegistrarTable*   registrarTable,
    assocID = selectRegistrarAssocID(registrarTable);
    if(assocID != 0) {
       /* ====== Registrar selected, get some information about it ======== */
-      n = getpaddrsplus(registrarFD, assocID, &peerAddressArray);
+      n = getpaddrsplus(registrarHuntFD, assocID, &peerAddressArray);
       if(n > 0) {
          LOG_VERBOSE2
-         fprintf(stdlog, "Assoc %u connected to registrar at ", assocID);
+         fprintf(stdlog, "Assoc %u connected to registrar at ", (unsigned int)assocID);
          fputaddress((struct sockaddr*)&peerAddressArray[0], true, stdlog);
          fputs("\n", stdlog);
          LOG_END
@@ -376,7 +395,8 @@ static int selectRegistrar(struct RegistrarTable*   registrarTable,
       }
 
       sd = registrarTablePeelOffRegistrarAssocID(registrarTable,
-                                                 registrarFD,
+                                                 registrarHuntFD,
+                                                 registrarHuntMessageBuffer,
                                                  assocID);
    }
    return(sd);
@@ -385,15 +405,16 @@ static int selectRegistrar(struct RegistrarTable*   registrarTable,
 
 /* ###### Peel off registrar assoc ####################################### */
 int registrarTablePeelOffRegistrarAssocID(struct RegistrarTable* registrarTable,
-                                          int                    registrarFD,
+                                          int                    registrarHuntFD,
+                                          struct MessageBuffer*  registrarHuntMessageBuffer,
                                           sctp_assoc_t           assocID)
 {
-   int sd = sctp_peeloff(registrarFD, assocID);
+   int sd = sctp_peeloff(registrarHuntFD, assocID);
    if(sd >= 0) {
       LOG_VERBOSE2
-      fprintf(stdlog, "Assoc %u peeled off from registrar hunt socket\n", assocID);
+      fprintf(stdlog, "Assoc %u peeled off from registrar hunt socket\n", (unsigned int)assocID);
       LOG_END
-      removeRegistrarAssocID(registrarTable, registrarFD, assocID);
+      removeRegistrarAssocID(registrarTable, registrarHuntFD, assocID);
    }
    return(sd);
 }
@@ -401,7 +422,8 @@ int registrarTablePeelOffRegistrarAssocID(struct RegistrarTable* registrarTable,
 
 /* ###### Handle notification on registrar hunt socket ################### */
 void registrarTableHandleNotificationOnRegistrarHuntSocket(struct RegistrarTable*         registrarTable,
-                                                           int                            registrarFD,
+                                                           int                            registrarHuntFD,
+                                                           struct MessageBuffer*          registrarHuntMessageBuffer,
                                                            const union sctp_notification* notification)
 {
    union sockaddr_union* peerAddressArray;
@@ -410,13 +432,13 @@ void registrarTableHandleNotificationOnRegistrarHuntSocket(struct RegistrarTable
    /* ====== Association change notification ============================= */
    if(notification->sn_header.sn_type == SCTP_ASSOC_CHANGE) {
       if(notification->sn_assoc_change.sac_state == SCTP_COMM_UP) {
-         n = getpaddrsplus(registrarFD,
+         n = getpaddrsplus(registrarHuntFD,
                            notification->sn_assoc_change.sac_assoc_id,
                            &peerAddressArray);
          if(n > 0) {
             LOG_VERBOSE2
             fprintf(stdlog, "Assoc %u connected to registrar at ",
-                    notification->sn_assoc_change.sac_assoc_id);
+                    (unsigned int)notification->sn_assoc_change.sac_assoc_id);
             fputaddress((struct sockaddr*)&peerAddressArray[0], true, stdlog);
             fputs("\n", stdlog);
             LOG_END
@@ -425,19 +447,28 @@ void registrarTableHandleNotificationOnRegistrarHuntSocket(struct RegistrarTable
 
          /* ====== Add registrar assoc ID to list ======================== */
          addRegistrarAssocID(registrarTable,
-                             registrarFD,
+                             registrarHuntFD,
                              notification->sn_assoc_change.sac_assoc_id);
       }
+      else if((notification->sn_assoc_change.sac_state == SCTP_COMM_LOST) ||
+              (notification->sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP) ) {
+         LOG_VERBOSE2
+         fprintf(stdlog, "Assoc %u disconnected from registrar (communication lost or shutdown complete)\n",
+                 (unsigned int)notification->sn_assoc_change.sac_assoc_id);
+         LOG_END
+         removeRegistrarAssocID(registrarTable,
+                                registrarHuntFD,
+                                notification->sn_assoc_change.sac_assoc_id);
+      }
    }
-   else if((notification->sn_assoc_change.sac_state == SCTP_COMM_LOST) ||
-           (notification->sn_header.sn_type == SCTP_SHUTDOWN_EVENT)) {
+   else if(notification->sn_header.sn_type == SCTP_SHUTDOWN_EVENT) {
       LOG_VERBOSE2
-      fprintf(stdlog, "Assoc %u disconnected from registrar\n",
-              notification->sn_assoc_change.sac_assoc_id);
+      fprintf(stdlog, "Assoc %u disconnected from registrar (shutdown)\n",
+              (unsigned int)notification->sn_shutdown_event.sse_assoc_id);
       LOG_END
       removeRegistrarAssocID(registrarTable,
-                             registrarFD,
-                             notification->sn_assoc_change.sac_assoc_id);
+                             registrarHuntFD,
+                             notification->sn_shutdown_event.sse_assoc_id);
    }
 }
 
@@ -451,8 +482,7 @@ void registrarTableDelete(struct RegistrarTable* registrarTable)
          ext_close(registrarTable->AnnounceSocket);
          registrarTable->AnnounceSocket = -1;
       }
-      leafLinkedRedBlackTreeDelete(&registrarTable->RegistrarAssocIDList);
-      ST_CLASS(peerListManagementClear)(&registrarTable->RegistrarList);
+      simpleRedBlackTreeDelete(&registrarTable->RegistrarAssocIDList);
       ST_CLASS(peerListManagementDelete)(&registrarTable->RegistrarList);
       free(registrarTable);
    }
@@ -460,21 +490,13 @@ void registrarTableDelete(struct RegistrarTable* registrarTable)
 
 
 /* ###### Add static registrar entry ################################### */
-unsigned int registrarTableAddStaticEntry(struct RegistrarTable* registrarTable,
-                                          union sockaddr_union*  addressArray,
-                                          size_t                 addresses)
+unsigned int registrarTableAddStaticEntry(
+                struct RegistrarTable*              registrarTable,
+                const struct TransportAddressBlock* transportAddressBlock)
 {
-   char                           transportAddressBlockBuffer[transportAddressBlockGetSize(MAX_PE_TRANSPORTADDRESSES)];
-   struct TransportAddressBlock*  transportAddressBlock = (struct TransportAddressBlock*)&transportAddressBlockBuffer;
    struct ST_CLASS(PeerListNode)* peerListNode;
-   int                            result;
+   unsigned int                   result;
 
-   transportAddressBlockNew(transportAddressBlock,
-                            IPPROTO_SCTP,
-                            getPort((struct sockaddr*)&addressArray[0]),
-                            0,
-                            addressArray,
-                            addresses);
    result = ST_CLASS(peerListManagementRegisterPeerListNode)(
                &registrarTable->RegistrarList,
                0,
@@ -502,7 +524,8 @@ unsigned int registrarTableAddStaticEntry(struct RegistrarTable* registrarTable,
 
 /* ###### Try more registrars ########################################### */
 static void tryNextBlock(struct RegistrarTable*         registrarTable,
-                         int                            registrarFD,
+                         int                            registrarHuntFD,
+                         struct MessageBuffer*          registrarHuntMessageBuffer,
                          RegistrarIdentifierType*       lastRegistrarIdentifier,
                          struct TransportAddressBlock** lastTransportAddressBlock)
 {
@@ -541,13 +564,13 @@ static void tryNextBlock(struct RegistrarTable*         registrarTable,
             transportAddressBlock = transportAddressBlock->Next;
          }
          if(transportAddressBlock != NULL) {
-            LOG_VERBOSE2
+            LOG_VERBOSE1
             fputs("Trying registrar at ",  stdlog);
             transportAddressBlockPrint(transportAddressBlock, stdlog);
             fputs("...\n",  stdlog);
             LOG_END
 
-            result = connectplus(registrarFD,
+            result = connectplus(registrarHuntFD,
                                  transportAddressBlock->AddressArray,
                                  transportAddressBlock->Addresses);
             if((result < 0) &&
@@ -577,11 +600,12 @@ static void tryNextBlock(struct RegistrarTable*         registrarTable,
 
 /* ###### Find registrar ################################################### */
 int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
-                               int                      registrarFD,
+                               int                      registrarHuntFD,
+                               struct MessageBuffer*    registrarHuntMessageBuffer,
                                RegistrarIdentifierType* registrarIdentifier)
 {
    struct timeval                 selectTimeout;
-   union sctp_notification        notification;
+   const union sctp_notification* notification;
    RegistrarIdentifierType        lastRegistrarIdentifier;
    struct TransportAddressBlock*  lastTransportAddressBlock;
    struct ST_CLASS(PeerListNode)* peerListNode;
@@ -597,18 +621,17 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
    int                            sd;
    int                            n;
 
-
    *registrarIdentifier = 0;
    if(registrarTable == NULL) {
-      return(0);
+      return(-1);
    }
-   LOG_VERBOSE
+   LOG_VERBOSE1
    fputs("Looking for registrar...\n",  stdlog);
    LOG_END
 
 
    /* ====== Is there already a connection to a registrar? =============== */
-   sd = selectRegistrar(registrarTable, registrarFD,
+   sd = selectRegistrar(registrarTable, registrarHuntFD, registrarHuntMessageBuffer,
                         registrarIdentifier);
    if(sd >= 0) {
       return(sd);
@@ -636,6 +659,7 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
 
    lastTrialTimeStamp = 0;
    for(;;) {
+      /* ====== Start new round ========================================== */
       if((lastRegistrarIdentifier == 0) && (lastTransportAddressBlock == NULL)) {
          /*
             Start new trial, when
@@ -657,7 +681,7 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                   transportAddressBlockDelete(lastTransportAddressBlock);
                   free(lastTransportAddressBlock);
                }
-               return(0);
+               return(-1);
             }
 
             lastTrialTimeStamp = 0;
@@ -678,19 +702,21 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
          }
       }
 
+      /* ====== Try next block of addresses ================================= */
       if(lastTrialTimeStamp + registrarTable->RegistrarConnectTimeout < getMicroTime()) {
          /* Try next block of addresses */
          tryNextBlock(registrarTable,
-                      registrarFD,
+                      registrarHuntFD,
+                      registrarHuntMessageBuffer,
                       &lastRegistrarIdentifier,
                       &lastTransportAddressBlock);
          lastTrialTimeStamp = getMicroTime();
       }
 
-      /* Wait for event */
+      /* ====== Wait for event =========================================== */
       FD_ZERO(&rfdset);
-      FD_SET(registrarFD, &rfdset);
-      n = registrarFD;
+      FD_SET(registrarHuntFD, &rfdset);
+      n = registrarHuntFD;
       if(registrarTable->AnnounceSocket >= 0) {
          FD_SET(registrarTable->AnnounceSocket, &rfdset);
          n = max(n, registrarTable->AnnounceSocket);
@@ -702,13 +728,6 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
       LOG_VERBOSE4
       fprintf(stdlog, "select() with timeout %lld\n", nextTimeout);
       LOG_END
-      /*
-         Important: rspSelect() may *not* be used here!
-         registrarTableFindRegistrar() may be called from within a
-         session timer + from within registrar socket callback
-         => Strange things will happen when these callbacks are
-            invoked recursively!
-      */
       result = ext_select(n + 1,
                           &rfdset, NULL, NULL,
                           &selectTimeout);
@@ -724,46 +743,54 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
             transportAddressBlockDelete(lastTransportAddressBlock);
             free(lastTransportAddressBlock);
          }
-         return(0);
+         return(-1);
       }
 
-      /* Handle events */
+      /* ====== Handle events ============================================ */
       if(result != 0) {
-         if(FD_ISSET(registrarTable->AnnounceSocket, &rfdset)) {
+         if( (registrarTable->AnnounceSocket > 0) &&
+             (FD_ISSET(registrarTable->AnnounceSocket, &rfdset)) ) {
             handleRegistrarAnnounceCallback(registrarTable, registrarTable->AnnounceSocket);
          }
-         if(FD_ISSET(registrarFD, &rfdset)) {
+         if(FD_ISSET(registrarHuntFD, &rfdset)) {
             flags    = MSG_PEEK;
-            received = recvfromplus(registrarFD,
-                                    &notification, sizeof(notification), &flags,
-                                    NULL, 0,
-                                    NULL, &assocID, NULL, 0);
+            received = messageBufferRead(registrarHuntMessageBuffer, registrarHuntFD, &flags,
+                                         NULL, 0,
+                                         NULL, &assocID, NULL, 0);
             if(received > 0) {
                if(flags & MSG_NOTIFICATION) {
+                  notification = (const union sctp_notification*)registrarHuntMessageBuffer->Buffer;
                   /* ====== Notification received ======================== */
                   flags = 0;
-                  if( (recvfromplus(registrarFD,
-                                    &notification, sizeof(notification), &flags,
-                                    NULL, 0,
-                                    NULL, NULL, NULL, 0) > 0) &&
-                     (flags & MSG_NOTIFICATION) ) {
-                        registrarTableHandleNotificationOnRegistrarHuntSocket(registrarTable,
-                                                                              registrarFD,
-                                                                              &notification);
-                        if((notification.sn_assoc_change.sac_state == SCTP_COMM_LOST) ||
-                           (notification.sn_assoc_change.sac_state == SCTP_CANT_STR_ASSOC) ||
-                           (notification.sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP)) {
-                           if(registrarTable->OutstandingConnects > 0) {
-                              registrarTable->OutstandingConnects--;
-                           }
-                           LOG_VERBOSE3
-                           fprintf(stdlog, "Failed to establish registrar connection, outstanding=%d\n", registrarTable->OutstandingConnects);
-                           LOG_END
+                  if( (messageBufferRead(registrarHuntMessageBuffer, registrarHuntFD, &flags,
+                                         NULL, 0,
+                                         NULL, NULL, NULL, 0) > 0) &&
+                      (flags & MSG_NOTIFICATION) ) {
+
+                     registrarTableHandleNotificationOnRegistrarHuntSocket(registrarTable,
+                                                                           registrarHuntFD,
+                                                                           registrarHuntMessageBuffer,
+                                                                           notification);
+                     if((notification->sn_assoc_change.sac_state == SCTP_COMM_LOST) ||
+                        (notification->sn_assoc_change.sac_state == SCTP_CANT_STR_ASSOC) ||
+                        (notification->sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP)) {
+                        if(registrarTable->OutstandingConnects > 0) {
+                           registrarTable->OutstandingConnects--;
                         }
+                        LOG_VERBOSE3
+                        fprintf(stdlog, "Failed to establish registrar connection, outstanding=%u\n",
+                                 (unsigned int)registrarTable->OutstandingConnects);
+                        LOG_END
+                     }
+                  }
+                  else {
+                     LOG_ERROR
+                     fputs("Peeked notification but failed to read it\n", stdlog);
+                     LOG_END_FATAL
                   }
 
                   /* ====== Is there a connection to a registrar? ======== */
-                  sd = selectRegistrar(registrarTable, registrarFD,
+                  sd = selectRegistrar(registrarTable, registrarHuntFD, registrarHuntMessageBuffer,
                                        registrarIdentifier);
                   if(sd >= 0) {
                      if(lastTransportAddressBlock) {
@@ -774,17 +801,30 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                   }
                }
                else {
-                  LOG_ERROR
-                  fputs("Peeked notification but failed to read it\n", stdlog);
-                  LOG_END_FATAL
+                  LOG_VERBOSE
+                  fprintf(stdlog, "Peeked data on assoc %u, but no association is listed\n",
+                          (unsigned int)assocID);
+                  LOG_END
+
+                  sd = registrarTablePeelOffRegistrarAssocID(registrarTable, registrarHuntFD, registrarHuntMessageBuffer,
+                                                             assocID);
+                  if(sd >= 0) {
+                     LOG_ACTION
+                     fprintf(stdlog, "Taking assoc %u as new registrar => socket %d\n", (unsigned int)assocID, sd);
+                     LOG_END
+                     return(sd);
+                  }
+
+                  LOG_WARNING
+                  fprintf(stdlog, "Got crap via assoc %u -> aborting it\n", (unsigned int)assocID);
+                  LOG_END
+                  flags = 0;
+                  messageBufferRead(registrarHuntMessageBuffer, registrarHuntFD, &flags,
+                                    NULL, 0,
+                                    NULL, &assocID, NULL, 0);
+                  sendabort(registrarHuntFD, assocID);
                }
             }
-            else {
-               LOG_ERROR
-               fprintf(stdlog, "Received data on assoc %u, but no association is listed\n",
-                       (unsigned int)assocID);
-               LOG_END_FATAL
-            }
          }
       }
    }
