@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: rserpoolerror.c 975 2006-03-01 21:11:38Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,11 +53,13 @@ static struct ErrorTable ErrorDescriptions[] = {
    { RSPERR_CONNECTION_FAILURE_SOCKET,     "socket() failure" },
    { RSPERR_CONNECTION_FAILURE_CONNECT,    "connect() failure" },
    { RSPERR_NO_REGISTRAR,                  "no registrar available" },
+   { RSPERR_TIMEOUT,                       "timeout" },
 
    /* Handlespace-management specific error causes */
    { RSPERR_NO_RESOURCES,                  "no resources" },
    { RSPERR_NOT_FOUND,                     "object not found" },
    { RSPERR_INVALID_ID,                    "invalid ID" },
+   { RSPERR_OWN_ID,                        "own ID" },
    { RSPERR_DUPLICATE_ID,                  "duplicate ID" },
    { RSPERR_WRONG_PROTOCOL,                "wrong protocol" },
    { RSPERR_WRONG_CONTROLCHANNEL_HANDLING, "wrong control channel handling" },
