@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: rserpoolerror.h 1181 2006-08-04 02:19:39Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,31 +41,33 @@ extern "C" {
 #define RSPERR_UNRECOGNIZED_MESSAGE          0x0002
 #define RSPERR_AUTHORIZATION_FAILURE         0x0003
 #define RSPERR_INVALID_VALUES                0x0004
+#define RSPERR_INCOMPATIBLE_POOL_POLICY      0x0005
+#define RSPERR_OUT_OF_MEMORY                 0x0006
+#define RSPERR_WRONG_PROTOCOL                0x0007
+#define RSPERR_WRONG_CONTROLCHANNEL_HANDLING 0x0008
+#define RSPERR_NOT_FOUND                     0x0009
 
 /* Implementation-specific error causes */
 #define RSPERR_NOT_INITIALIZED               0x1000
 #define RSPERR_BUFFERSIZE_EXCEEDED           0x1001
-#define RSPERR_OUT_OF_MEMORY                 0x1002
 #define RSPERR_READ_ERROR                    0x1003
 #define RSPERR_WRITE_ERROR                   0x1004
 #define RSPERR_CONNECTION_FAILURE_SOCKET     0x1005
 #define RSPERR_CONNECTION_FAILURE_CONNECT    0x1006
 #define RSPERR_NO_REGISTRAR                  0x1007
+#define RSPERR_TIMEOUT                       0x1008
 
 /* Handlespace-management specific error causes */
 #define RSPERR_NO_RESOURCES                  0xf002
-#define RSPERR_NOT_FOUND                     0xf003
 #define RSPERR_INVALID_ID                    0xf004
-#define RSPERR_DUPLICATE_ID                  0xf005
-#define RSPERR_WRONG_PROTOCOL                0xf006
-#define RSPERR_WRONG_CONTROLCHANNEL_HANDLING 0xf007
-#define RSPERR_INCOMPATIBLE_POOL_POLICY      0xf008
-#define RSPERR_INVALID_POOL_POLICY           0xf009
-#define RSPERR_INVALID_POOL_HANDLE           0xf00a
-#define RSPERR_INVALID_ADDRESSES             0xf00b
-#define RSPERR_INVALID_REGISTRATOR           0xf00c
-#define RSPERR_NO_USABLE_ASAP_ADDRESSES      0xf00d
-#define RSPERR_NO_USABLE_USER_ADDRESSES      0xf00e
+#define RSPERR_OWN_ID                        0xf005
+#define RSPERR_DUPLICATE_ID                  0xf006
+#define RSPERR_INVALID_POOL_POLICY           0xf00a
+#define RSPERR_INVALID_POOL_HANDLE           0xf00b
+#define RSPERR_INVALID_ADDRESSES             0xf00c
+#define RSPERR_INVALID_REGISTRATOR           0xf00d
+#define RSPERR_NO_USABLE_ASAP_ADDRESSES      0xf00e
+#define RSPERR_NO_USABLE_USER_ADDRESSES      0xf00f
 
 
 const char* rserpoolErrorGetDescription(const unsigned int error);
