@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessage.h 713 2005-08-17 15:31:56Z dreibh $
+ *  $Id: rserpoolmessage.h 1091 2006-05-09 08:48:20Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -79,6 +79,8 @@ extern "C" {
 #define AHT_ERROR                      (0x0e | AHT_ASAP_MODIFIER)
 
 
+#define AHF_REGISTRATION_REJECT        (1 << 0)
+
 #define AHF_ENDPOINT_KEEP_ALIVE_HOME   (1 << 0)
 
 
@@ -167,6 +169,14 @@ struct rserpool_policy_leastused
    uint32_t pp_lu_load:24;
 };
 
+struct rserpool_policy_leastused_dpf
+{
+   uint8_t  pp_ludpf_policy:8;
+   uint32_t pp_ludpf_load:24;
+   uint32_t pp_ludpf_load_dpf;
+   uint32_t pp_ludpf_distance;
+};
+
 struct rserpool_policy_leastused_degradation
 {
    uint8_t  pp_lud_policy:8;
@@ -175,6 +185,16 @@ struct rserpool_policy_leastused_degradation
    uint32_t pp_lud_loaddeg:24;
 };
 
+struct rserpool_policy_leastused_degradation_dpf
+{
+   uint8_t  pp_luddpf_policy:8;
+   uint32_t pp_luddpf_load:24;
+   uint8_t  pp_luddpf_pad:8;
+   uint32_t pp_luddpf_loaddeg:24;
+   uint32_t pp_luddpf_load_dpf;
+   uint32_t pp_luddpf_distance;
+};
+
 struct rserpool_policy_priority_leastused
 {
    uint8_t  pp_plu_policy:8;
@@ -201,6 +221,14 @@ struct rserpool_policy_weighted_random
    uint32_t pp_wrd_weight:24;
 };
 
+struct rserpool_policy_weighted_random_dpf
+{
+   uint8_t  pp_wrddpf_policy:8;
+   uint32_t pp_wrddpf_weight:24;
+   uint32_t pp_wrddpf_weight_dpf;
+   uint32_t pp_wrddpf_distance;
+};
+
 struct rserpool_policy_randomized_leastused
 {
    uint8_t  pp_rlu_policy:8;
@@ -351,11 +379,12 @@ struct RSerPoolMessage
 
    struct ST_CLASS(PeerListNode)*              PeerListNodePtr;
    bool                                        PeerListNodePtrAutoDelete;
-   struct ST_CLASS(PeerList)*                  PeerListPtr;
+   struct ST_CLASS(PeerListManagement)*        PeerListPtr;
    bool                                        PeerListPtrAutoDelete;
 
    struct ST_CLASS(PoolHandlespaceManagement)* HandlespacePtr;
    bool                                        HandlespacePtrAutoDelete;
+   size_t                                      MaxElementsPerHTRequest;
 
    struct ST_CLASS(HandleTableExtract)*        ExtractContinuation;
 
