@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessagecreator.c 697 2005-08-08 09:26:18Z dreibh $
+ *  $Id: rserpoolmessagecreator.c 1091 2006-05-09 08:48:20Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -294,11 +294,14 @@ static bool createPolicyParameter(struct RSerPoolMessage*          message,
    struct rserpool_policy_roundrobin*                                rr;
    struct rserpool_policy_weighted_roundrobin*                       wrr;
    struct rserpool_policy_leastused*                                 lu;
+   struct rserpool_policy_leastused_dpf*                             ludpf;
    struct rserpool_policy_leastused_degradation*                     lud;
+   struct rserpool_policy_leastused_degradation_dpf*                 luddpf;
    struct rserpool_policy_priority_leastused*                        plu;
    struct rserpool_policy_priority_leastused_degradation*            plud;
    struct rserpool_policy_random*                                    rd;
    struct rserpool_policy_weighted_random*                           wrd;
+   struct rserpool_policy_weighted_random_dpf*                       wrddpf;
    struct rserpool_policy_randomized_leastused*                      rlu;
    struct rserpool_policy_randomized_leastused_degradation*          rlud;
    struct rserpool_policy_randomized_priority_leastused*             rplu;
@@ -332,6 +335,16 @@ static bool createPolicyParameter(struct RSerPoolMessage*          message,
           wrd->pp_wrd_policy = poolPolicySettings->PolicyType;
           wrd->pp_wrd_weight = hton24(poolPolicySettings->Weight);
        break;
+      case PPT_WEIGHTED_RANDOM_DPF:
+          wrddpf = (struct rserpool_policy_weighted_random_dpf*)getSpace(message, sizeof(struct rserpool_policy_weighted_random_dpf));
+          if(wrddpf == NULL) {
+             return(false);
+          }
+          wrddpf->pp_wrddpf_policy     = poolPolicySettings->PolicyType;
+          wrddpf->pp_wrddpf_weight     = hton24(poolPolicySettings->Weight);
+          wrddpf->pp_wrddpf_weight_dpf = htonl(poolPolicySettings->WeightDPF);
+          wrddpf->pp_wrddpf_distance   = htonl(poolPolicySettings->Distance);
+       break;
       case PPT_ROUNDROBIN:
           rr = (struct rserpool_policy_roundrobin*)getSpace(message, sizeof(struct rserpool_policy_roundrobin));
           if(rr == NULL) {
@@ -356,6 +369,16 @@ static bool createPolicyParameter(struct RSerPoolMessage*          message,
           lu->pp_lu_policy = poolPolicySettings->PolicyType;
           lu->pp_lu_load   = hton24(poolPolicySettings->Load);
        break;
+      case PPT_LEASTUSED_DPF:
+          ludpf = (struct rserpool_policy_leastused_dpf*)getSpace(message, sizeof(struct rserpool_policy_leastused_dpf));
+          if(ludpf == NULL) {
+             return(false);
+          }
+          ludpf->pp_ludpf_policy   = poolPolicySettings->PolicyType;
+          ludpf->pp_ludpf_load     = hton24(poolPolicySettings->Load);
+          ludpf->pp_ludpf_load_dpf = htonl(poolPolicySettings->LoadDPF);
+          ludpf->pp_ludpf_distance = htonl(poolPolicySettings->Distance);
+       break;
       case PPT_LEASTUSED_DEGRADATION:
           lud = (struct rserpool_policy_leastused_degradation*)getSpace(message, sizeof(struct rserpool_policy_leastused_degradation));
           if(lud == NULL) {
@@ -366,6 +389,18 @@ static bool createPolicyParameter(struct RSerPoolMessage*          message,
           lud->pp_lud_pad     = 0x00;
           lud->pp_lud_loaddeg = hton24(poolPolicySettings->LoadDegradation);
        break;
+      case PPT_LEASTUSED_DEGRADATION_DPF:
+          luddpf = (struct rserpool_policy_leastused_degradation_dpf*)getSpace(message, sizeof(struct rserpool_policy_leastused_degradation_dpf));
+          if(luddpf == NULL) {
+             return(false);
+          }
+          luddpf->pp_luddpf_policy   = poolPolicySettings->PolicyType;
+          luddpf->pp_luddpf_load     = hton24(poolPolicySettings->Load);
+          luddpf->pp_luddpf_pad      = 0x00;
+          luddpf->pp_luddpf_loaddeg  = hton24(poolPolicySettings->LoadDegradation);
+          luddpf->pp_luddpf_load_dpf = htonl(poolPolicySettings->LoadDPF);
+          luddpf->pp_luddpf_distance = htonl(poolPolicySettings->Distance);
+       break;
       case PPT_PRIORITY_LEASTUSED:
           plu = (struct rserpool_policy_priority_leastused*)getSpace(message, sizeof(struct rserpool_policy_priority_leastused));
           if(plu == NULL) {
@@ -636,6 +671,14 @@ static bool createEndpointKeepAliveMessage(struct RSerPoolMessage* message)
       return(false);
    }
 
+   /* ------ Custom parameter ------ */
+   if(message->Identifier != 0) {
+      if(createPoolElementIdentifierParameter(message, message->Identifier) == false) {
+         return(false);
+      }
+   }
+   /* ------------------------------ */
+
    return(finishMessage(message));
 }
 
@@ -713,7 +756,7 @@ static bool createRegistrationResponseMessage(struct RSerPoolMessage* message)
    }
 
    if(beginMessage(message, AHT_REGISTRATION_RESPONSE,
-                   message->Flags & 0x00, PPID_ASAP) == NULL) {
+                   message->Flags & AHF_REGISTRATION_REJECT, PPID_ASAP) == NULL) {
       return(false);
    }
 
@@ -974,7 +1017,7 @@ static bool createListResponseMessage(struct RSerPoolMessage* message)
    sp->sp_receiver_id = htonl(message->ReceiverID);
 
    peerListNode = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(
-                     message->PeerListPtr);
+                     &message->PeerListPtr->List);
    while(peerListNode != NULL) {
       /* The draft does not say what to do when the amount of peers exceeds the
          message size -> We fill as much entries as possible and reply this
@@ -985,7 +1028,7 @@ static bool createListResponseMessage(struct RSerPoolMessage* message)
          break;
       }
       peerListNode = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(
-                        message->PeerListPtr,
+                        &message->PeerListPtr->List,
                         peerListNode);
    }
 
@@ -1028,7 +1071,7 @@ static bool createHandleTableResponseMessage(struct RSerPoolMessage* message)
    struct rserpool_header*              header;
 
    header = beginMessage(message, EHT_HANDLE_TABLE_RESPONSE,
-                         message->Flags & (EHF_HANDLE_TABLE_RESPONSE_REJECT|EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND),
+                         message->Flags & EHF_HANDLE_TABLE_RESPONSE_REJECT,
                          PPID_ENRP);
    if(header == NULL) {
       return(false);
@@ -1057,8 +1100,7 @@ static bool createHandleTableResponseMessage(struct RSerPoolMessage* message)
       result = ST_CLASS(poolHandlespaceManagementGetHandleTable)(
                   message->HandlespacePtr,
                   message->HandlespacePtr->Handlespace.HomeRegistrarIdentifier,
-                  hte,
-                  flags);
+                  hte,flags, message->MaxElementsPerHTRequest);
       if(result > 0) {
          lastPoolHandle = NULL;
          for(i = 0;i < hte->PoolElementNodes;i++) {
@@ -1075,6 +1117,7 @@ static bool createHandleTableResponseMessage(struct RSerPoolMessage* message)
                      return(false);
                   }
                   message->Position = oldPosition;
+                  header->ah_flags |= EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND;
                   break;
                }
             }
@@ -1085,6 +1128,7 @@ static bool createHandleTableResponseMessage(struct RSerPoolMessage* message)
                   return(false);
                }
                message->Position = oldPosition;
+               header->ah_flags |= EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND;
                break;
             }
             oldPosition = message->Position;
@@ -1092,13 +1136,13 @@ static bool createHandleTableResponseMessage(struct RSerPoolMessage* message)
             hte->LastPoolHandle            = hte->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
             hte->LastPoolElementIdentifier = hte->PoolElementNodeArray[i]->Identifier;
          }
-         if((hte->PoolElementNodes == NTE_MAX_POOL_ELEMENT_NODES) ||
+         if((hte->PoolElementNodes == min(message->MaxElementsPerHTRequest, NTE_MAX_POOL_ELEMENT_NODES)) ||
             (i != hte->PoolElementNodes)) {
             header->ah_flags |= EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND;
          }
       }
 
-      if(hte->PoolElementNodes < NTE_MAX_POOL_ELEMENT_NODES) {
+      if(!(header->ah_flags & EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND)) {
          free(message->PeerListNodePtr->UserData);
          message->PeerListNodePtr->UserData = NULL;
       }
