@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessageparser.c 697 2005-08-08 09:26:18Z dreibh $
+ *  $Id: rserpoolmessageparser.c 1175 2006-08-03 00:27:08Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -8,7 +8,7 @@
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
  * This program is free software; you can redistribute it and/or
@@ -127,7 +127,7 @@ static bool handleUnknownTLV(struct RSerPoolMessage* message,
       return(false);
    }
    else if((tlvType & ATT_ACTION_MASK) == ATT_ACTION_STOP) {
-      LOG_VERBOSE3
+      LOG_VERBOSE1
       fprintf(stdlog, "Silently stop processing for type $%02x at position %u\n",
               tlvType, (unsigned int)message->Position);
       LOG_END
@@ -136,7 +136,7 @@ static bool handleUnknownTLV(struct RSerPoolMessage* message,
       return(false);
    }
    else if((tlvType & ATT_ACTION_MASK) == ATT_ACTION_STOP_AND_REPORT) {
-      LOG_VERBOSE3
+      LOG_VERBOSE1
       fprintf(stdlog, "Stop processing for type $%02x at position %u\n",
               tlvType, (unsigned int)message->Position);
       LOG_END
@@ -154,7 +154,7 @@ static size_t checkBeginMessage(struct RSerPoolMessage* message,
                                 size_t*                 startPosition)
 {
    struct rserpool_header* header;
-   size_t              length;
+   size_t                  length;
 
    *startPosition                     = message->Position;
    message->OffendingMessageTLV       = (char*)&message->Buffer[*startPosition];
@@ -235,6 +235,8 @@ static size_t checkBeginTLV(struct RSerPoolMessage* message,
    uint16_t                    tlvType;
    size_t                      tlvLength;
 
+   tlvLength = 0;
+   tlvType   = 0;
    while(getNextTLV(message, tlvPosition, &header, &tlvType, &tlvLength)) {
       if((!checkType) || (PURE_ATT_TYPE(tlvType) == (expectedType))) {
          break;
@@ -291,7 +293,7 @@ static bool checkFinishTLV(struct RSerPoolMessage* message,
 
 /* ###### Scan address parameter ######################################### */
 static bool scanAddressParameter(struct RSerPoolMessage* message,
-                                 const card16            port,
+                                 const uint16_t          port,
                                  union sockaddr_union*   address)
 {
    struct sockaddr_in*  in;
@@ -504,11 +506,14 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
    struct rserpool_policy_roundrobin*                                rr;
    struct rserpool_policy_weighted_roundrobin*                       wrr;
    struct rserpool_policy_leastused*                                 lu;
+   struct rserpool_policy_leastused_dpf*                             ludpf;
    struct rserpool_policy_leastused_degradation*                     lud;
+   struct rserpool_policy_leastused_degradation_dpf*                 luddpf;
    struct rserpool_policy_priority_leastused*                        plu;
    struct rserpool_policy_priority_leastused_degradation*            plud;
    struct rserpool_policy_random*                                    rd;
    struct rserpool_policy_weighted_random*                           wrd;
+   struct rserpool_policy_weighted_random_dpf*                       wrddpf;
    struct rserpool_policy_randomized_leastused*                      rlu;
    struct rserpool_policy_randomized_leastused_degradation*          rlud;
    struct rserpool_policy_randomized_priority_leastused*             rplu;
@@ -554,6 +559,32 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
             return(false);
          }
        break;
+      case PPT_LEASTUSED_DPF:
+         if(tlvLength >= sizeof(struct rserpool_policy_leastused_dpf)) {
+            ludpf = (struct rserpool_policy_leastused_dpf*)getSpace(message, sizeof(struct rserpool_policy_leastused_dpf));
+            if(ludpf == NULL) {
+               return(false);
+            }
+            poolPolicySettings->PolicyType = ludpf->pp_ludpf_policy;
+            poolPolicySettings->Weight     = 0;
+            poolPolicySettings->Load       = ntoh24(ludpf->pp_ludpf_load);
+            poolPolicySettings->LoadDPF    = ntohl(ludpf->pp_ludpf_load_dpf);
+            poolPolicySettings->Distance   = ntohl(ludpf->pp_ludpf_distance);
+            LOG_VERBOSE3
+            fprintf(stdlog, "Scanned policy LU-DPF, load=$%06x, ldpf=%1.3f%%, distance=%u\n",
+                    poolPolicySettings->Load,
+                    100.0 * ((double)poolPolicySettings->LoadDPF / (double)0xffffffff),
+                    poolPolicySettings->Distance);
+            LOG_END
+         }
+         else {
+            LOG_WARNING
+            fputs("LU-DPF TLV too short\n", stdlog);
+            LOG_END
+            message->Error = RSPERR_INVALID_VALUES;
+            return(false);
+         }
+       break;
       case PPT_LEASTUSED_DEGRADATION:
          if(tlvLength >= sizeof(struct rserpool_policy_leastused_degradation)) {
             lud = (struct rserpool_policy_leastused_degradation*)getSpace(message, sizeof(struct rserpool_policy_leastused_degradation));
@@ -578,6 +609,34 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
             return(false);
          }
        break;
+      case PPT_LEASTUSED_DEGRADATION_DPF:
+         if(tlvLength >= sizeof(struct rserpool_policy_leastused_degradation_dpf)) {
+            luddpf = (struct rserpool_policy_leastused_degradation_dpf*)getSpace(message, sizeof(struct rserpool_policy_leastused_degradation_dpf));
+            if(luddpf == NULL) {
+               return(false);
+            }
+            poolPolicySettings->PolicyType      = luddpf->pp_luddpf_policy;
+            poolPolicySettings->Weight          = 0;
+            poolPolicySettings->Load            = ntoh24(luddpf->pp_luddpf_load);
+            poolPolicySettings->LoadDegradation = ntoh24(luddpf->pp_luddpf_loaddeg);
+            poolPolicySettings->LoadDPF         = ntohl(luddpf->pp_luddpf_load_dpf);
+            poolPolicySettings->Distance        = ntohl(luddpf->pp_luddpf_distance);
+            LOG_VERBOSE3
+            fprintf(stdlog, "Scanned policy LUD-DPF, load=$%06x, loaddeg=$%06x, ldpf=%1.3f%%, distance=%u\n",
+                    poolPolicySettings->Load,
+                    poolPolicySettings->LoadDegradation,
+                    100.0 * ((double)poolPolicySettings->LoadDPF / (double)0xffffffff),
+                    poolPolicySettings->Distance);
+            LOG_END
+         }
+         else {
+            LOG_WARNING
+            fputs("LUD-DPF TLV too short\n", stdlog);
+            LOG_END
+            message->Error = RSPERR_INVALID_VALUES;
+            return(false);
+         }
+       break;
       case PPT_PRIORITY_LEASTUSED:
          if(tlvLength >= sizeof(struct rserpool_policy_priority_leastused)) {
             plu = (struct rserpool_policy_priority_leastused*)getSpace(message, sizeof(struct rserpool_policy_priority_leastused));
@@ -658,7 +717,7 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
             poolPolicySettings->Load       = 0;
 
             LOG_VERBOSE3
-            fprintf(stdlog, "Scanned policy WRAND, weight=%d\n",
+            fprintf(stdlog, "Scanned policy WRAND, weight=%u\n",
                     poolPolicySettings->Weight);
             LOG_END
          }
@@ -670,6 +729,34 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
             return(false);
          }
        break;
+      case PPT_WEIGHTED_RANDOM_DPF:
+         if(tlvLength >= sizeof(struct rserpool_policy_weighted_random_dpf)) {
+            wrddpf = (struct rserpool_policy_weighted_random_dpf*)getSpace(message, sizeof(struct rserpool_policy_weighted_random_dpf));
+            if(wrddpf == NULL) {
+               message->Error = RSPERR_INVALID_VALUES;
+               return(false);
+            }
+            poolPolicySettings->PolicyType = wrddpf->pp_wrddpf_policy;
+            poolPolicySettings->Weight     = ntoh24(wrddpf->pp_wrddpf_weight);
+            poolPolicySettings->Load       = 0;
+            poolPolicySettings->WeightDPF  = ntohl(wrddpf->pp_wrddpf_weight_dpf);
+            poolPolicySettings->Distance   = ntohl(wrddpf->pp_wrddpf_distance);
+
+            LOG_VERBOSE3
+            fprintf(stdlog, "Scanned policy WRAND-DPF, weight=%u, wdpf=%1.3f%%, distance=%u\n",
+                    poolPolicySettings->Weight,
+                    100.0 * ((double)poolPolicySettings->WeightDPF / (double)0xffffffff),
+                    poolPolicySettings->Distance);
+            LOG_END
+         }
+         else {
+            LOG_WARNING
+            fputs("WRAND-DPF TLV too short\n", stdlog);
+            LOG_END
+            message->Error = RSPERR_INVALID_VALUES;
+            return(false);
+         }
+       break;
       case PPT_ROUNDROBIN:
          if(tlvLength >= sizeof(struct rserpool_policy_roundrobin)) {
             rr = (struct rserpool_policy_roundrobin*)getSpace(message, sizeof(struct rserpool_policy_roundrobin));
@@ -703,7 +790,7 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
             poolPolicySettings->Weight     = ntoh24(wrr->pp_wrr_weight);
             poolPolicySettings->Load       = 0;
             LOG_VERBOSE3
-            fprintf(stdlog, "Scanned policy WRR, weight=%d\n",poolPolicySettings->Weight);
+            fprintf(stdlog, "Scanned policy WRR, weight=%u\n",poolPolicySettings->Weight);
             LOG_END
          }
          else {
@@ -1171,6 +1258,13 @@ static bool scanEndpointKeepAliveMessage(struct RSerPoolMessage* message)
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
       return(false);
    }
+
+   /* ------ Custom parameter ------ */
+   if(scanPoolElementIdentifierParameter(message) == false) {
+      message->Identifier = 0;
+   }
+   /* ------------------------------ */
+
    return(true);
 }
 
@@ -1448,6 +1542,7 @@ static bool scanListResponseMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
    struct ST_CLASS(PeerListNode)*   peerListNode;
+   struct ST_CLASS(PeerListNode)*   newPeerListNode;
    unsigned int                     errorCode;
 
    sp = (struct rserpool_serverparameter*)getSpace(message, sizeof(struct rserpool_serverparameter));
@@ -1466,17 +1561,27 @@ static bool scanListResponseMessage(struct RSerPoolMessage* message)
          }
 
          if(message->PeerListPtr == NULL) {
-            message->PeerListPtr = (struct ST_CLASS(PeerList)*)malloc(sizeof(struct ST_CLASS(PeerList)));
+            message->PeerListPtr = (struct ST_CLASS(PeerListManagement)*)malloc(sizeof(struct ST_CLASS(PeerListManagement)));
             if(message->PeerListPtr == NULL) {
                message->Error = RSPERR_OUT_OF_MEMORY;
                free(peerListNode);
                return(false);
             }
-            ST_CLASS(peerListNew)(message->PeerListPtr, 0);
+            ST_CLASS(peerListManagementNew)(message->PeerListPtr, 0, 0, NULL, NULL);
          }
-         ST_CLASS(peerListAddPeerListNode)(message->PeerListPtr,
-                                           peerListNode,
-                                           &errorCode);
+
+         peerListNode->Flags |= PLNF_FROM_PEER;
+         if(peerListNode->Identifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
+            peerListNode->Flags |= PLNF_DYNAMIC;
+         }
+
+         errorCode = ST_CLASS(peerListManagementRegisterPeerListNode)(
+                        message->PeerListPtr,
+                        peerListNode->Identifier,
+                        peerListNode->Flags,
+                        peerListNode->AddressBlock,
+                        0,
+                        &newPeerListNode);
          if(errorCode != RSPERR_OKAY) {
             LOG_WARNING
             fputs("ListResponse contains bad peer ", stdlog);
@@ -1485,10 +1590,16 @@ static bool scanListResponseMessage(struct RSerPoolMessage* message)
             rserpoolErrorPrint(errorCode, stdlog);
             fputs("\n", stdlog);
             LOG_END
+            ST_CLASS(peerListNodeDelete)(peerListNode);
             free(peerListNode);
             message->Error = (uint16_t)errorCode;
             return(false);
          }
+
+         free(peerListNode->AddressBlock);
+         peerListNode->AddressBlock = NULL;
+         ST_CLASS(peerListNodeDelete)(peerListNode);
+         free(peerListNode);
       }
    }
 
@@ -1764,8 +1875,9 @@ static bool scanMessage(struct RSerPoolMessage* message)
    }
    else {
       LOG_ERROR
-      fprintf(stdlog, "Wrong PPID ($%08x)\n", message->PPID);
-      LOG_END_FATAL
+      fprintf(stdlog, "Wrong PPID ($%08x), expected ASAP or ENRP\n", message->PPID);
+      LOG_END
+      return(false);
    }
 
    switch(message->Type) {
@@ -2026,8 +2138,7 @@ unsigned int rserpoolPacket2Message(char*                       packet,
       fputs("\n", stdlog);
       LOG_END
 
-      CHECK((*message)->Error != RSPERR_OKAY);
       return((*message)->Error);
    }
-   return(RSPERR_NO_RESOURCES);
+   return(RSPERR_OUT_OF_MEMORY);
 }
