@@ -1,6 +1,8 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2006 by Thomas Dreibholz
+ *
+ * $Id: transportaddressblock.c 1091 2006-05-09 08:48:20Z dreibh $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -450,7 +452,7 @@ size_t transportAddressBlockFilter(
       filteredAddressBlock->Next      = NULL;
       filteredAddressBlock->Protocol  = originalAddressBlock->Protocol;
       filteredAddressBlock->Port      = originalAddressBlock->Port;
-      filteredAddressBlock->Flags     = 0;
+      filteredAddressBlock->Flags     = originalAddressBlock->Flags;
       filteredAddressBlock->Addresses = selected;
       j = 0;
       for(i = 0;i < originalAddressBlock->Addresses;i++) {
